<?php

/**
 * Prodigy admin attributes class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-product-attributes.php';
include_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-prodigy-admin-notification.php';


/**
 * Class Prodigy_Admin_Attributes_Menu
 */
class Prodigy_Admin_Attributes_Menu {
    
    const ATTRIBUTES_PAGE = 'prodigy_attributes';
    
    const ACTION_CREATE = 'create';
    const ACTION_UPDATE = 'update';
    
    
    const ORDER_NAME = 'Name';
    const ORDER_NAME_NUMERIC = 'Name (numeric)';
    const ORDER_TERM_ID = 'Term ID';
    
    public static $order_type_mapper = [
        0 => self::ORDER_NAME,
        1 => self::ORDER_NAME_NUMERIC,
        2 => self::ORDER_TERM_ID
    ];
    
    public $product_attributes;
    
    /**
     * Prodigy_Admin_Menu constructor.
     */
    public function __construct() {
        add_action( 'admin_menu', [ $this, 'add_prodigy_attributes_menu' ], 10 );
        add_action( 'admin_enqueue_scripts', [ $this, 'load_attribute_menu_scripts' ] );
        add_action( 'wp_ajax_create-prodigy-attribute', [ $this, 'prodigy_create_attribute' ] );
        add_action( 'wp_ajax_delete-prodigy-attribute', [ $this, 'prodigy_delete_attribute' ] );
        
        $this->product_attributes = new Prodigy_Product_Attributes();
    }
    
    /**
     * Load page scripts
     */
    public function load_attribute_menu_scripts() {
        wp_enqueue_script( 'jquery-ui-autocomplete' );
    }
    
    /**
     * Create attributes submenu
     */
    public function add_prodigy_attributes_menu() {
        
        $attribute_menu = add_submenu_page(
            'edit.php?post_type=' . Prodigy::get_prodigy_product_type(),
            __( 'Prodigy Attributes', 'prodigy' ),
            __( 'Attributes', 'prodigy' ),
            'edit_pages',
            self::ATTRIBUTES_PAGE,
            [ $this, 'get_attributes_page' ]
        );
        
        add_action( 'load-' . $attribute_menu, [ $this, 'prodigy_attribute_page_init' ] );
    }
    
    /**
     *  Save page params
     */
    public function prodigy_attribute_page_init() {
        
        if ( isset( $_POST['update_attribute'] ) ) {
            
            $request_id = (int) $_GET['id'];
            
            $count_products = Prodigy_Product_Attributes::get_count_variants_from_attribute( $request_id );
            
            if ( ! empty( $count_products ) ) {
                wp_redirect( admin_url( 'edit.php?post_type='.Prodigy::get_prodigy_product_type().'&page=prodigy_attributes' ) );
                Prodigy_Admin_Notification::displayError( esc_html__( 'Attribute you want to edit is used as an option for ' . $count_products . ' product(s). Please, disable "use as option" for this attribute on all related products', 'prodigy' ) );
                exit();
            }
            $is_update = $this->product_attributes->update_attribute_with_values_by_id( $request_id, $_POST );
            if ( $is_update ) {
                wp_redirect( admin_url( 'edit.php?post_type='.Prodigy::get_prodigy_product_type().'&page=prodigy_attributes' ) );
                Prodigy_Admin_Notification::displaySuccess( esc_html__( 'Attribute successfully updated!', 'prodigy' ) );
                exit();
            }
        }
    }
    
    /**
     * Delete attribute from ajax
     */
    public function prodigy_delete_attribute() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $result = false;
        
        $attr_slug = $_POST['attr_slug'] ?? sanitize_title( $_POST['attr_slug'] ) ?? '';
        
        if ( $attr_slug ) {
            /**
             * clear metadata _pg_attributes for product
             */
            $this->product_attributes->delete_relation_attr_by_slug_for_products( $attr_slug );
            
            $result = $this->product_attributes->delete_attribute_by_slug( $attr_slug );
        }
        
        $message_error = esc_html__( 'Can`t delete attribute', 'prodigy' );
        
        if ( ! empty( $result ) ) {
            echo wp_json_encode( [
                'error'  => $message_error,
                'status' => 0,
            ] );
        } else {
            echo wp_json_encode( [
                'status' => '',
            ] );
            Prodigy_Admin_Notification::displaySuccess( esc_html__( 'Attribute ' . $attr_slug . ' has been deleted!', 'prodigy' ) );
        }
        
        wp_die();
    }
    
    /**
     *  Save attribute from ajax
     */
    public function prodigy_create_attribute() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $request_slug  = $_POST['slug'] ?? sanitize_title( $_POST['slug'] ) ?? '';
        $request_label = $_POST['label'] ?? sanitize_title( $_POST['label'] ) ?? '';
        $request_order = $_POST['order'] ?? (int) $_POST['order'] ?? 0;
        
        $response = '';
        $slug     = empty( $request_slug ) ? $request_label : $request_slug;
        
        $array_values = [];
        if ( isset( $_POST['values'] ) && ! empty( $_POST['values'] ) ) {
            $array_values = explode( ",", filter_var( $_POST['values'], FILTER_SANITIZE_STRIPPED ) );
        }
        
        if ( ! empty( $request_label ) && ! empty( $slug ) && ! empty( $array_values ) ) {
            $response = $this->product_attributes->save( $request_label, strtolower( $slug ), $array_values, $request_order );
        }
        
        if ( $response !== '' ) {
            
            $error_message   = esc_html__( "Name or Slug " . $request_label . " is already in use. Change it, please.", 'prodigy' );
            $success_message = esc_html__( 'Attribute successfully created!', 'prodigy' );
            
            if ( ! $response['result'] ) {
                echo wp_json_encode( [
                    'error'  => $error_message,
                    'status' => 0,
                ] );
                Prodigy_Admin_Notification::displayError( $error_message );
                
            } else {
                echo wp_json_encode( [
                    'attribute' => $response['current_attribute'],
                    'status'    => $response['result'],
                ] );
                Prodigy_Admin_Notification::displaySuccess( $success_message );
            }
        }
        
        wp_die();
    }
    
    /**
     * Init the attributes page.
     */
    public function get_attributes_page() {
        // update action
        if ( isset( $_GET['action'] ) ) {
            $this->render_view();
        } else {
            $search = isset($_POST['query_search']) ? sanitize_text_field( $_POST['query_search'] ) : '';
            $this->render_view( $search );
        }
        
    }
    
    /**
     * Render view from page
     *
     * @param string $search
     */
    public function render_view( $search = '' ) {
        
        $request_page   = isset($_GET['page']) ? sanitize_title( $_GET['page'] ) : '';
        $request_id     = isset($_GET['id']) ? (int) $_GET['id'] : 0;
        $request_action = isset($_GET['action']) ? $_GET['action'] : '';
        
        if ( $request_page === Prodigy_Admin_Attributes_Menu::ATTRIBUTES_PAGE && ! $request_action ) {
            $attributes = $this->product_attributes->get_all_attributes_with_values_list( $search );
            require_once PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-attributes-create-menu-page.php';
        } elseif ( $request_page === Prodigy_Admin_Attributes_Menu::ATTRIBUTES_PAGE && $request_action ) {
            $item = [];
            if ( $request_id ) {
                $item         = $this->product_attributes->get_attribute_with_values_by_id( $request_id );
                $terms_values = implode( ',', wp_list_pluck( $item['values'], 'name' ) );
            }
            require_once PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-attributes-edit-menu-page.php';
        }
    }
}

return new Prodigy_Admin_Attributes_Menu();
