<?php

/**
 * Prodigy admin categories class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Taxonomy
 */
class Prodigy_Admin_Category extends Prodigy_Main_Abstract {
    
    private $hosted_category_id;
    const PRODIGY_HOSTED_CATEGORY_RELATION = 'prodigy_hosted_category_relation';

    const PRODIGY_HOSTED_SEND_SUCCESS = 'success';
    const PRODIGY_HOSTED_SEND_FAILED = 'failed';

    /**
     * Category constructor.
     */
    public function __construct() {
        parent::__construct();
        
        $category_type = Prodigy::get_prodigy_category_type();
        
        add_action( $category_type.'_add_form_fields', [ $this, 'add_category_image' ], 10, 2 );
        add_action( 'created_'.$category_type, [ $this, 'save_category_image' ], 10, 2 );
        add_action( 'created_'.$category_type, [ $this, 'save_order' ], 10, 2 );
        add_action( $category_type.'_edit_form_fields', [ $this, 'update_category_image' ], 10, 2 );
        add_action( 'edited_'.$category_type, [ $this, 'updated_category_image' ], 10, 2 );
        add_action( 'admin_footer', [ $this, 'add_script' ] );
        
        // add columns in grid
        add_filter( 'manage_edit-'.$category_type.'_columns', [ $this, 'product_cat_columns' ] );
        add_filter( 'manage_'.$category_type.'_custom_column', [ $this, 'product_cat_column' ], 10, 3 );

        add_action( 'wp_ajax_retry-synchronize-category', [ $this, 'add_category_to_hosted' ] );
        
        if ( ! empty( get_option( 'pg_store_key' ) ) ) {
            add_action( 'created_'.$category_type, [ $this, 'add_relationship_categories' ], 12, 2 );
            add_action( 'created_'.$category_type, [ $this, 'add_category_to_hosted' ], 11, 2 );
            add_action( 'pre_delete_term', [ $this, 'delete_category_from_hosted' ], 10, 2 );
            add_action( 'edit_terms', [ $this, 'update_category_from_hosted' ], 10, 2 );
        }

        add_action( 'admin_enqueue_scripts', [ $this, 'wptutsplus_post_listing_column_resize'] );
    }

    public function wptutsplus_post_listing_column_resize() { ?>
        <style type="text/css">
            .column-status {
                width: 10% !important;
            }
        </style>
    <?php }
    
    /**
     * @param $term_id
     * @param $taxonomy
     *
     * @return bool
     */
    public function update_category_from_hosted( $term_id, $taxonomy ) {
        
        if ( $_POST['action'] != 'editedtag' ) {
            return true;
        }
        
        if ( $taxonomy != Prodigy::get_prodigy_category_type() ) {
            return true;
        }
        
        $name   = filter_var( $_POST['name'], FILTER_SANITIZE_STRIPPED );
        $parent = filter_var( $_POST['parent'], FILTER_VALIDATE_INT );
        
        $hosted_category_id        = get_term_meta( $term_id, self::PRODIGY_HOSTED_CATEGORY_RELATION, true );
        $hosted_parent_category_id = get_term_meta( $parent, self::PRODIGY_HOSTED_CATEGORY_RELATION, true );
        
        $result = $this->api_client->update_category( $name, (int) $hosted_category_id, (int) $hosted_parent_category_id );
        
        if ( array_key_exists( 'error', $result ) ) {
            wp_die( $result['message'] );
        }
        
        $status_code = wp_remote_retrieve_response_code( $result );


        if ( $status_code != 200 ) {
            $errors  = $this->api_client->get_list_errors( $result );
            $message = wp_remote_retrieve_response_message( $result ) . '. ' . implode( ', ', $errors );
            
            wp_die( $message );
        }
        
        return true;
    }
    
    /**
     * @param $term
     * @param $taxonomy
     *
     * @return bool
     */
    public function delete_category_from_hosted( $term, $taxonomy ) {
        
        /*
         * delete only categories
         */
        if ( $taxonomy != Prodigy::get_prodigy_category_type() ) {
            return true;
        }
        
        $hosted_category_id = get_term_meta( $term, self::PRODIGY_HOSTED_CATEGORY_RELATION, true );
        
        if ( empty( $hosted_category_id ) ) {
            //wp_die( esc_html__( 'Error by delete category', 'prodigy' ) );
            return true;
        }
        
        $result = $this->api_client->delete_category( (int) $hosted_category_id );
        
        if ( array_key_exists( 'error', $result ) ) {
            wp_die( $result['message'] );
        }
        
        $status_code = wp_remote_retrieve_response_code( $result );
        
        if ( $status_code != 204 ) {
            $errors  = $this->api_client->get_list_errors( $result );
            $message = wp_remote_retrieve_response_message( $result ) . '. ' . implode( ', ', $errors );
            if ( $status_code == 401 ) {
                $message = esc_html__( 'The category wasn\'t deleted due to connection issues', 'prodigy' );
            }
            
            wp_die( $message );
        }
    }
    
    /**
     * Add relationship from hosted system to termmeta with term_id
     *
     * @param $term_id
     * @param $term_taxonomy_id
     */
    public function add_relationship_categories( $term_id, $term_taxonomy_id ) {
        add_term_meta( $term_id, self::PRODIGY_HOSTED_CATEGORY_RELATION, $this->hosted_category_id, true );
    }
    
    /**
     *
     * @param $term_id
     *
     * @return string
     */
    public function add_category_to_hosted( $term_id ) {
    
//        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $term_id = $_POST['term_id'] ?? $term_id;

        $current_type_category = Prodigy::get_prodigy_category_type();
        
        $list_valid_actions = [ 'add-'.$current_type_category, 'add-tag', 'retry-synchronize-category' ];
    
        if ( ! in_array( sanitize_text_field($_POST['action']), $list_valid_actions ) ) {
            return true;
        }
        
        $info_new_category = get_term($term_id);
        
        $name   = filter_var( $info_new_category->name, FILTER_SANITIZE_STRIPPED );
        $parent = filter_var( $info_new_category->parent, FILTER_VALIDATE_INT );
        
        $hosted_parent_category_id = get_term_meta( $parent, self::PRODIGY_HOSTED_CATEGORY_RELATION, true );

        $result = $this->api_client->create_category( $name, (int) $hosted_parent_category_id );
        
        if ( array_key_exists( 'error', $result ) ) {
            wp_die( $result['message'] );
        }
        
        $status_code = wp_remote_retrieve_response_code( $result );
        
        if ( $status_code != 201 ) {
            $errors  = $this->api_client->get_list_errors( $result );
            $message = wp_remote_retrieve_response_message( $result ) . '. ' . implode( ', ', $errors );
            
            if ( $status_code == 401 ) {
                $message = esc_html__( 'The category wasn\'t created due to connection issues', 'prodigy' );
            }
            
            wp_delete_term($term_id, Prodigy::get_prodigy_category_type());
            
            wp_die( $message );
        } else {
            $result_body              = json_decode( wp_remote_retrieve_body( $result ) );
            $this->hosted_category_id = $result_body->data->id;
            update_term_meta( $term_id, self::PRODIGY_HOSTED_CATEGORY_RELATION, $this->hosted_category_id, true );
        }

    }
    
    /**
     * Add a form field in the new category page
     *
     * @param string $taxonomy
     */
    public function add_category_image( $taxonomy ) {
        require_once PRODIGY_PLUGIN_PATH . 'admin/partials/add-category-image.php';
    }
    
    /**
     * @param int $term_id
     */
    public function save_category_image( $term_id ) {
        if ( isset( $_POST['category-image-id'] ) && '' !== $_POST['category-image-id'] ) {
            $image = sanitize_text_field($_POST['category-image-id']);
            add_term_meta( $term_id, 'category-image-id', $image, true );
        }
    }
    
    /**
     * @param int $term_id
     */
    public function save_order( $term_id ) {
        add_term_meta( $term_id, 'order', 0, true );
    }
    
    /**
     * Edit the form field
     *
     * @param object $term
     */
    public function update_category_image( $term ) {
        require_once PRODIGY_PLUGIN_PATH . 'admin/partials/update-category-image.php';
    }
    
    /**
     * @param int $term_id
     */
    public function updated_category_image( $term_id ) {
        if ( isset( $_POST['category-image-id'] ) && '' !== $_POST['category-image-id'] ) {
            $image = sanitize_text_field($_POST['category-image-id']);
            update_term_meta( $term_id, 'category-image-id', $image );
        } else {
            update_term_meta( $term_id, 'category-image-id', '' );
        }
    }
    
    /*
     * Add script
     */
    public function add_script() {
        wp_enqueue_script(
            'upload_category',
            dirname( plugin_dir_url( __FILE__ ) ) . '/web/admin/js/upload-image.js',
            [
                'jquery',
                'jquery-ui-core',
                'jquery-ui-sortable'
            ],
            false,
            true
        );
    }
    
    /**
     * Thumbnail column added to category admin.
     *
     * @param mixed $columns
     *
     * @return array
     */
    public function product_cat_columns( $columns ) {
        $new_columns = [];
        
        if ( isset( $columns['cb'] ) ) {
            $new_columns['cb'] = $columns['cb'];
            unset( $columns['cb'] );
        }
        
        $new_columns['thumb'] = 'Image';
        $new_columns['status'] = 'Status';

        if ( isset( $columns['name'] ) ) {
            $new_columns['name'] = 'Name';
            unset( $columns['name'] );
        }

        if ( isset( $columns['description'] ) ) {
            $new_columns['handle_desc'] = 'Description';
            unset( $columns['description'] );
        }

        $columns = array_merge( $new_columns, $columns );
        
        if ( ! wp_is_mobile() ) {
            $columns['handle'] = 'Sort';
        }

        $n_columns = $this->add_column_after( $columns, 'name', 'thumb' );
        $n_columns = $this->add_column_after( $n_columns, 'handle_desc', 'status' );

        return $n_columns;
    }


    public function add_column_after($columns, $move, $before) {
        $n_columns = [];
        foreach ( $columns as $key => $value ) {
            if ( $key === $before ) {
                $n_columns[ $move ] = $move;
            }
            $n_columns[ $key ] = $value;
        }

        return $n_columns;
    }
    
    /**
     * Thumbnail column value added to category
     *
     * @param string $columns
     * @param string $column
     * @param int $id
     *
     * @return string
     */
    public function product_cat_column( $columns, $column, $id ) {
        if ( 'thumb' === $column ) {
            $thumbnail_id = get_term_meta( $id, 'category-image-id', true );
            
            if ( $thumbnail_id ) {
                $image = wp_get_attachment_thumb_url( $thumbnail_id );
            } else {
                $image = plugin_dir_url( PRODIGY_PLUGIN_PATH . 'web/admin/images/placeholder.png' ) . 'placeholder.png';
            }
            
            $image   = str_replace( ' ', '%20', $image );
            $columns .= '<img src="' . $image . '" alt="Thumb' . '" class="wp-post-image" height="48" width="48" />';
        }
        if ( 'handle' === $column ) {
            $columns .= '<i class="fas fa-bars"></i>';
            $columns .= '<input type="hidden" name="term_id" value="' . esc_attr( $id ) . '" />';
        }
        
        if ( 'handle_desc' === $column ) {
            $term        = get_term( $id );
            $description = sanitize_text_field( $term->description );
            if ( strlen( $description ) > 130 ) {
                $textPrev = substr( $description, 0, 130 );
                $textPrev = rtrim( $textPrev, "!,.-" );
                $textPrev = substr( $textPrev, 0, strrpos( $textPrev, ' ' ) );
                $textNext = substr( $description, strlen( $textPrev ) );
                $columns  .= '<div class="review-text"><div class="text-prev">' . esc_attr($textPrev) . '</div><div class="text-next" style="display: none">' . esc_attr($textNext) . '</div><a href="#" class="text-more row-title">&nbsp;&#8230;</a></div>';
            } else {
                $columns .= '<div class="review-text">' . $description . '</div>';
            }
        }

        if ( 'status' === $column ) {
            if (get_term($id)->name !== Prodigy::PRODIGY_DEFAULT_NAME_CATEGORY) {
                if ( !empty(get_term_meta($id, self::PRODIGY_HOSTED_CATEGORY_RELATION)) ) {
                    echo "<span style='color: green;font-weight: bold'>" . esc_html__( self::PRODIGY_HOSTED_SEND_SUCCESS,
                            'prodigy' ) . "</span>";
                } else {
                    echo "<span style='color: red;font-weight: bold' class='retry'>" . esc_html__( self::PRODIGY_HOSTED_SEND_FAILED,
                            'prodigy' ) . "</span>
                            <a href='#' id='category_synchronize' data-id=" . esc_attr( $id ) . ">retry</a>";
                }
            }
        }
        
        return $columns;
    }
    
}
