<?php

include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-api-client.php';

/**
 * Prodigy admin settings class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Admin_Menu
 */
class Prodigy_Admin_Settings_Menu extends Prodigy_Main_Abstract {
    /*
    * Tab actions
    */
    const ACTION_GENERAL = 'general';
    const ACTION_PRODUCTS = 'product';

    /*
     * Menu slug
     */
    const SETTINGS_PAGE = 'prodigy_settings';

    /**
     * Update messages.
     *
     * @var array
     */
    private static $messages = [];

    /**
     * Error messages.
     *
     * @var array
     */
    private static $errors = [];
    
    const CURRENT_SHOP_PAGE_ID = 0;

    /**
     * Checkbox settings
     *
     * @var array
     */
    public static $default_checkbox_fields = [
        'pg_inventory_hide_products',
        'pg_product_review',
        'pg_product_rating',
        'pg_captcha_launch'
    ];

    /**
     * Prodigy_Admin_Menu constructor.
     */
    public function __construct() {

        parent::__construct();
        add_action( 'admin_menu', [ $this, 'add_prodigy_settings_menu' ], 11 );
        add_action( 'wp_ajax_connect-store-prodigy', [ $this, 'connect_to_store' ] );
        add_action( 'wp_ajax_change-store-key-prodigy', [ $this, 'change_store_key' ] );
        add_action( 'wp_ajax_disconnect-store-key-prodigy', [ $this, 'disconnect_store_key' ] );
        add_action( 'wp_ajax_update-store-info', [ $this, 'update_info_store' ] );
        add_action( 'wp_ajax_validate-type-url', [ $this, 'is_valid_url_name_ajax' ], 10, 1 );
        add_action( 'admin_init', [$this, 'update_types'], 999 );
        add_action( 'admin_init', [$this, 'update_types_category'], 999 );
        add_action( 'admin_init', [$this, 'update_types_tag'], 999 );
    }
    
    /**
     *
     */
    public function update_types() {
        global $wpdb;
        
        $all_types_old = get_option(Prodigy::PRODIGY_OPTION_NAME_OLD_PRODUCT_TYPES);
        $access_update_option = get_option( 'pg_access_update_type_products_queue' );
        $update_process = new Prodigy_Update_Type_Products();
        
        $result = $wpdb->get_results(
            "SELECT `option_name`
                   FROM $wpdb->options
                   WHERE `option_name` LIKE '%pg_update_type_product_queue_batch%';"
        );
        
        if ( ! empty( $access_update_option ) && empty( $result ) ) {
            
            $list_products  = get_posts( [ 'post_type' => $all_types_old, 'posts_per_page' => -1 ] );
            
            if ( ! empty( $list_products ) && is_array( $list_products )  ) {
                
                foreach ( $list_products as $key => $val ) {

                    $update_process->push_to_queue( $val->ID );
                }
                $update_process->save()->dispatch();
            }
        }
    }
    
    
    /**
     * Check and run process update terms category
     */
    public function update_types_category() {
        
        $access_update_option = get_option( 'pg_access_update_type_categories_queue' );
        
        if ( ! empty( $access_update_option ) ) {
            ( new Prodigy_Update_Type_Categories() )->update();
        }
    }
    
    /**
     * Check and run process update terms category
     */
    public function update_types_tag() {
        
        $access_update_option = get_option( 'pg_access_update_type_tags_queue' );
        
        if ( ! empty( $access_update_option ) ) {
            ( new Prodigy_Update_Type_Tags() )->update();
        }
    }
    
    /**
     * @param $term
     *
     * @return array
     */
    public function custom_get_terms( $term ) {
        global $wpdb;

        $out = array();

        $a = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT t.name,t.slug,t.term_group,x.term_taxonomy_id,x.term_id,x.taxonomy,x.description,x.parent,x.count
                            FROM {$wpdb->prefix}term_taxonomy x LEFT JOIN {$wpdb->prefix}terms t ON (t.term_id = x.term_id)
                            WHERE x.taxonomy=%s;",
                    $term
                )
        );

        foreach ( $a as $b ) {
            $obj                   = new stdClass();
            $obj->term_id          = $b->term_id;
            $obj->name             = $b->name;
            $obj->slug             = $b->slug;
            $obj->term_group       = $b->term_group;
            $obj->term_taxonomy_id = $b->term_taxonomy_id;
            $obj->taxonomy         = $b->taxonomy;
            $obj->description      = $b->description;
            $obj->parent           = $b->parent;
            $obj->count            = $b->count;
            $out[]                 = $obj;
        }

        return $out;
    }
    
    /**
     * Update info store from hosted system
     */
    public function update_info_store() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );

        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );

        
        $pg_store_key = filter_var( $_POST['pg_store_key'], FILTER_SANITIZE_STRIPPED );
        
        $subdomain = get_option('pg_url_domain_hosted_system');
    
        if ( empty( $subdomain ) ) {
            wp_send_json_error( esc_html__( 'No connection', 'prodigy' ), 422 );
        }
        
        $request_store = $this->api_client->get_store( $subdomain, $pg_store_key );
        
        if ( array_key_exists( 'error', $request_store ) ) {
            wp_send_json_error( $request_store['message'], $request_store['code'] );
        }
        
        $status_code = wp_remote_retrieve_response_code( $request_store );

        if ( $status_code == 200 ) {
            $result_body = json_decode( wp_remote_retrieve_body( $request_store ) );
            
            update_option( 'pg_domain_hosted_system', $result_body->data->attributes->name );
            update_option( 'pg_url_domain_hosted_system', $result_body->data->attributes->subdomain );
            
            $data['pg_domain_hosted_system'] = get_option('pg_domain_hosted_system');
            $data['pg_url_domain_hosted_system'] = esc_url(Prodigy_Helper_Hosted_System::get_url_home());
            $data['message'] = esc_html__( 'Data updated successfully', 'prodigy' );
            
            wp_send_json_success( $data );
            
        } else {
            $errors  = $this->api_client->get_list_errors( $request_store );
            $message = wp_remote_retrieve_response_message( $request_store ) . '. ' . implode( ', ', $errors );
            
            wp_send_json_error( $message, $status_code );
        }
        
        wp_die();
    }
    
    /**
     * connect to hosted system
     */
    public function connect_to_store() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );

        $pg_store_key = filter_var( $_POST['pg_store_key'], FILTER_SANITIZE_STRIPPED );

        $request_connection = $this->api_client->setup_connection( $pg_store_key, site_url() );

        if ( array_key_exists( 'error', $request_connection ) ) {
            wp_send_json_error( $request_connection['message'], $request_connection['code'] );
        }

        $status_code = wp_remote_retrieve_response_code( $request_connection );
        if ( $status_code == 204 ) {

            update_option( 'pg_store_key', $pg_store_key, false );

            $message = esc_html__( 'Connected successfully', 'prodigy' );
            wp_send_json_success( $message );

        } else {
            $errors  = $this->api_client->get_list_errors( $request_connection );
            $message = wp_remote_retrieve_response_message( $request_connection ) . '. ' . implode( ', ', $errors );

            wp_send_json_error( $message, $status_code );
        }

        wp_die();
    }

    /**
     *
     */
    public function change_store_key() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );

        /*
         * here change if need
         */

        $this->connect_to_store();
    }


    /**
     * @return bool
     */
    public function set_default_category() {

        $slug = sanitize_title( Prodigy::PRODIGY_DEFAULT_NAME_CATEGORY );

        $term_info = term_exists( $slug, Prodigy::get_prodigy_category_type() );

        if ( empty( $term_info['term_id'] ) ) {
            wp_send_json_error( esc_html__( 'Category default does not exist', 'prodigy' ), 422 );
        }

        $hosted_category_id = get_term_meta( $term_info['term_id'],
            Prodigy_Admin_Category::PRODIGY_HOSTED_CATEGORY_RELATION, true );

        if ( ! empty( $hosted_category_id ) ) {
            return true;
        }

        $new_connection_api = new Prodigy_Api_Client();

        $result = $new_connection_api->create_category( $slug );

        $status_code = wp_remote_retrieve_response_code( $result );

        if ( array_key_exists( 'error', $result ) ) {
            wp_send_json_error( $result['message'], $status_code );
        }

        if ( $status_code != 201 ) {
            $errors  = $this->api_client->get_list_errors( $result );
            $message = wp_remote_retrieve_response_message( $result ) . '. ' . implode( ', ', $errors );

            wp_send_json_error( $message, $status_code );
        } else {

            $result_body = json_decode( wp_remote_retrieve_body( $result ) );

            add_term_meta( $term_info['term_id'], Prodigy_Admin_Category::PRODIGY_HOSTED_CATEGORY_RELATION,
                $result_body->data->id, true );
        }

        return true;
    }


    /**
     * Disconnected from hosted system
     */
    public function disconnect_store_key() {
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $request_disconnection = $this->api_client->disconnection();

        if ( array_key_exists( 'error', $request_disconnection ) ) {
            wp_send_json_error( $request_disconnection['message'], $request_disconnection['code'] );
        }

        $status_code = wp_remote_retrieve_response_code( $request_disconnection );

        if ( $status_code == 204 ) {
            update_option( 'pg_store_key', '' );
            $message = esc_html__( 'Disconnected successfully', 'prodigy' );

            wp_send_json_success( $message );

        } else {

            $errors  = $this->api_client->get_list_errors( $request_disconnection );
            $message = wp_remote_retrieve_response_message( $request_disconnection ) . '. ' . implode( ', ', $errors );

            wp_send_json_error( $message, $status_code );
        }

        wp_die();
    }

    /**
     * @param bool $show
     */
    public function update_out_stock_products( $show = true ) {
        $select_status = $show ? Prodigy::PRODIGY_HIDE_STATUS : 'publish';
        $update_status = $show ? 'publish' : Prodigy::PRODIGY_HIDE_STATUS;
        $posts         = $this->get_products_for_meta_key( 'product_stock_status', 1, $select_status );
        $ids           = [];
        foreach ( $posts as $key => $post ) {
            $ids[ $key ] = $post->ID;
        }

        $ids = empty( $ids ) ? "''" : implode( ",", $ids );

        $this->update_product_status( $ids, $update_status );
    }

    /**
     * @param string $meta_key
     * @param mixed  $value
     * @param string $status
     *
     * @return array|null
     */
    public function get_products_for_meta_key( $meta_key, $value, $status ) {
        global $wpdb;

        $type = Prodigy::get_prodigy_product_type();

        return $wpdb->get_results(
            $wpdb->prepare("SELECT $wpdb->posts.*
                        FROM $wpdb->posts
                        LEFT JOIN $wpdb->postmeta
                          ON $wpdb->postmeta.post_id = $wpdb->posts.ID
                        WHERE $wpdb->posts.post_type = %s
                        AND $wpdb->posts.post_status = %s
                        AND $wpdb->postmeta.meta_key = %s
                        AND $wpdb->postmeta.meta_value = %d ",
                [ $type, $status, $meta_key, $value ]
            )
        );
    }

    /**
     * @param string $post_ids
     * @param string $status
     *
     * @return array
     */
    public function update_product_status( $post_ids, $status ) {
        global $wpdb;

        return $wpdb->get_results(
            $wpdb->prepare(
                "UPDATE $wpdb->posts SET post_status = %s WHERE ID IN ($post_ids)",
                [ $status ]
            )
        );

    }

    /**
     * @param array $post
     */
    public function apply_project_settings( $post ) {
        $this->update_out_stock_products( ! isset( $post['pg_inventory_hide_products'] ) );
    }

    /**
     * Create settings submenu
     */
    public function add_prodigy_settings_menu() {

        $menu_setting_page = add_submenu_page(
            'edit.php?post_type=' . Prodigy::get_prodigy_product_type(),
            esc_html__( 'Prodigy Settings', 'prodigy' ),
            esc_html__( 'Settings', 'prodigy' ),
            'edit_pages',
            self::SETTINGS_PAGE,
            [ $this, 'get_settings_page' ]
        );

        add_action( 'load-' . $menu_setting_page, [ $this, 'prodigy_settings_page_init' ] );
    }

    /**
     * Save prodigy settings
     */
    public function prodigy_settings_page_init() {
        global $tab_current;
        
        $request_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : '';

        $tab_current = empty( $request_tab ) ? self::ACTION_GENERAL : sanitize_title( wp_unslash( $request_tab ) );

        $this->save();
    }

    /**
     * Init the attributes page.
     */
    public function get_settings_page() {
        self::display_page();
    }

    /**
     * Display menu page
     */
    public static function display_page() {
        self::render_view();
    }

    /**
     * Render view for page
     */
    public static function render_view() {
        
        $is_connected = (bool) get_option( 'pg_store_key' );
        $domain_hosted_system     = get_option( 'pg_domain_hosted_system', 'Store name domain hosted' );
        
        if ( $_GET['page'] === self::SETTINGS_PAGE ) {
            require_once PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-admin-settings-page.php';
        }
    }
    
    /**
     * @return bool
     */
    public function is_validation() {
        
        $result = true;
        
        $prodigy_product_type_slug = $_POST['prodigy_product_type_slug'];
        $prodigy_category_type_slug = $_POST['prodigy_category_type_slug'];
        $prodigy_tag_type_slug = $_POST['prodigy_tag_type_slug'];
        
        if ( ! $this->is_valid_url_name($prodigy_product_type_slug)) {
            return false;
        }
    
        if ( ! $this->is_valid_url_name($prodigy_category_type_slug)) {
            return false;
        }
    
        if ( ! $this->is_valid_url_name($prodigy_tag_type_slug)) {
            return false;
        }
        
        return $result;
    }
    
    /**
     * @param $slug
     *
     * @return bool
     */
    public function is_valid_url_name( $slug ) {
        $result = true;
        
        if ( empty( $slug ) ) {
            $slug = $_POST['slug'];
        }
        
        $mask_url = '/^[0-9a-zA-Z_-]{2,}$/';
        
        if ( ! preg_match( $mask_url, $slug ) ) {
            return false;
        } elseif ( ! Prodigy_Type::is_valid_name_slug( $slug ) ) {
            return false;
        } elseif ($slug !== sanitize_title($slug)) {
            return false;
        }
        
        return $result;
    }
    
    /**
     * validation for ajax
     */
    public function is_valid_url_name_ajax() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
        
        $slug = $_POST['slug'];
        if ( $this->is_valid_url_name( $_POST['slug'] ) ) {
            $message = esc_html__( 'Slug is valid', 'prodigy' );
            wp_send_json_success( $message );
        } else {
            $message = esc_html__( 'Slug '.$slug.' is not valid', 'prodigy' );
            wp_send_json_error( $message, 422 );
        }
    }

    /**
     * Save the settings.
     */
    public function save() {
        global $tab_current;

        if ( empty( $tab_current ) ) {
            return false;
        }
        
        $request_page = $_GET['page'] ?? sanitize_text_field($_GET['page']) ?? '';

        if ( ! empty( $_POST ) && 'product' === $tab_current && $request_page === self::SETTINGS_PAGE  ) {
            
            if ( ! $this->is_validation() ) {
                Prodigy_Admin_Notification::displayError( esc_html__( 'Url name is not valid', 'prodigy' ) );
                return false;
            }
            
            if ( $this->save_fields( $_POST ) ) {
                Prodigy_Admin_Notification::displaySuccess( esc_html__( 'Your settings have been saved', 'prodigy' ) );
            }

            if ( isset( $_POST ) && $request_page == self::SETTINGS_PAGE ) {
                $this->apply_project_settings( $_POST );
            }

        }

    }

    /**
     * @param array $settings
     *
     * @return bool
     */
    public function save_fields( $settings ) {
        $result      = false;
        $field_names = self::$default_checkbox_fields;

        foreach ( $field_names as $option ) {
            if ( ! isset( $settings[ $option ] ) ) {
                if ( delete_option( sanitize_text_field($option) ) ) {
                    $result = true;
                }
            }
        }

        foreach ( $settings as $name => $value ) {
            if ( update_option( $name, $value ) ) {
                $result = true;
            }
        }
        
        wp_cache_flush();
        
        $current_product_type = Prodigy::get_prodigy_product_type();
        $new_product_type     = get_option( 'prodigy_product_type_slug' );
    
        $current_category_type = Prodigy::get_prodigy_category_type();
        $new_category_type     = get_option( 'prodigy_category_type_slug' );
    
        $current_tag_type = Prodigy::get_prodigy_tag_type();
        $new_tag_type     = get_option( 'prodigy_tag_type_slug' );
    
        $redirect_setting = false;
    
        if ( ! empty($new_product_type) && $current_product_type != $new_product_type ) {
            wp_cache_flush();
            ( new Prodigy_Type( Prodigy::PRODIGY_POST_TYPE_DEFAULT ) )->change_prodigy_type( $current_product_type, $new_product_type );
            $redirect_setting = true;
        }
    
        if ( ! empty($new_category_type) && $current_category_type != $new_category_type ) {
            wp_cache_flush();
            ( new Prodigy_Type( Prodigy::PRODIGY_TAXONOMY_CATEGORY_DEFAULT ) )->change_prodigy_type( $current_category_type, $new_category_type );
            $redirect_setting = true;
        }
    
        if ( ! empty($new_tag_type) && $current_tag_type != $new_tag_type ) {
            wp_cache_flush();
            ( new Prodigy_Type( Prodigy::PRODIGY_TAXONOMY_TAG_DEFAULT ) )->change_prodigy_type( $current_tag_type, $new_tag_type );
            $redirect_setting = true;
        }
    
        if ( $redirect_setting ) {
            
            Prodigy_Type::redirect_to_settings( empty($new_product_type) ? $current_product_type : $new_product_type );
        }

        return $result;
    }

    /**
     * Add a message.
     *
     * @param string $text Message.
     */
    public static function add_message( $text ) {
        self::$messages[] = $text;
    }

    /**
     * Add an error.
     *
     * @param string $text Message.
     */
    public static function add_error( $text ) {
        self::$errors[] = $text;
    }
    
    /**
     * List of pages for setting store display
     *
     * @return array|false
     */
    public static function get_post_pages() {
        return get_pages();
    }

}

return new Prodigy_Admin_Settings_Menu();
