<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @version 1.0.0
 * @package prodigy/admin
 */


defined( 'ABSPATH' ) || exit;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 */
class Prodigy_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	public $cart;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.1
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
        $this->cart = new Prodigy_Cart();

        // Maintain hierarchy of terms.
        add_filter( 'wp_terms_checklist_args', array( $this, 'disable_checked_ontop' ) );
        
        add_action( 'in_admin_header', [ $this, 'screen_option_menus' ] );
        add_action( 'wp_ajax_update-current-migration', [ $this, 'update_current_migration' ] );
        add_action( 'wp_ajax_prodigy-check-product', [ $this, 'prodigy_is_deleted_product' ] );
	}


    /**
     * Ajax - check deleted product
     */
	public function prodigy_is_deleted_product() {
        global $wpdb;
        
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
        
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );

        if (isset($_POST['product_id'])) {
            $product = get_post($_POST['product_id']);

            if ($product->post_parent) {
                $parent_product = get_post($product->post_parent);
                $status = $parent_product->post_status;
            } else {
                $status = $product->post_status;
            }

            if ($status === 'publish') {
                wp_send_json_success([1], 200);
            } else {
                $this->cart->remove_item_from_local_cart( $_POST['product_id'], $_COOKIE['prodigy_user'] );
                wp_send_json_error([0]);
            }

            wp_die();
        }
    }

    /**
     * ajax execute current migration
     */
	public function update_current_migration() {
        
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
        
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        if ( ProdigyUpdaterMigration::checkCurrentVersionMigration() ) {
            $info_migration = new ProdigyUpdaterMigration();
            $result = $info_migration->executeMigration();
        }
        
        if ( isset($result) ) {
            $message = esc_html__( 'Migration was successful', 'prodigy' );
            wp_send_json_success( $message );
        }
        
        $message = esc_html__( 'Migration failed', 'prodigy' );
        wp_send_json_error( $message, 422 );
    }

    /**
     * Maintain term hierarchy when editing a product.
     *
     * @param  array $args Term checklist args.
     * @return array
     */
    public function disable_checked_ontop( $args ) {
        if ( ! empty( $args['taxonomy'] ) && Prodigy::get_prodigy_category_type() === $args['taxonomy'] ) {
            $args['checked_ontop'] = false;
        }
        return $args;
    }
    
    /**
     * We clear screen option of unnecessary tags
     */
    public function screen_option_menus() {
        global $wp_meta_boxes;
        
        if ( isset( get_current_screen()->id ) && get_current_screen()->id == 'nav-menus' ) {
            
            $list            = Prodigy_Product_Attributes::get_attribute_taxonomies();
            $def_list_option = $wp_meta_boxes[ get_current_screen()->id ]['side']['default'];
            if ( ! empty( $list ) ) {
                foreach ( $list as $key => $value ) {
                    $slug = 'add-' . $value->slug;
                    if ( array_key_exists( $slug, $def_list_option ) ) {
                        unset( $wp_meta_boxes[ get_current_screen()->id ]['side']['default'][ $slug ] );
                    }
                }
            }
        }
    }

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
        $screen = get_current_screen();
        $current_page = $screen ? $screen->id : '';
        
        if ( isset( $_GET['page'] ) ) {
            if ( $_GET['page'] != 'prodigy-setup' ) {
                wp_enqueue_style( 'wp_prodigy_plugin', plugin_dir_url( __FILE__ ) . 'assets/css/admin.css', [], $this->version );
            }
        } else {
            wp_enqueue_style( 'wp_prodigy_plugin', plugin_dir_url( __FILE__ ) . 'assets/css/admin.css', [], $this->version );
        }

        if ( in_array( $current_page, Prodigy::get_pages_list() ) ) {
            wp_enqueue_style(
                'fontawesome-all',
                dirname( plugin_dir_url( __FILE__ ) ) . '/web/libraries/css/fontawesome-all.css',
                [],
                $this->version
            );
            wp_enqueue_style( 'font-style' );
            wp_enqueue_style( 'wp-jquery-ui-dialog' );
        }
    }


	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
	    $screen = get_current_screen();
        $current_page = $screen ? $screen->id : '';

        if ( in_array( $current_page, Prodigy::get_pages_list() ) ) {

            wp_enqueue_script(
                'admin',
                plugin_dir_url( __FILE__ ) . 'assets/js/admin.js',
                [ 'jquery' ],
                $this->version,
                false
            );

            wp_localize_script(
                'admin',
                'plugin_path_dir',
                [ 'dir' => dirname( plugin_dir_url(__FILE__) ) . '/web/' ]
            );

            // Term ordering - only when sorting by term_order.
            if ( ( ( ! empty( $_GET['taxonomy'] ) ) ) && ! isset( $_GET['orderby'] ) ) {
                wp_register_script(
                    'prodigy_term_ordering',
                    dirname( plugin_dir_url(__FILE__) ) . '/web/admin/js/term-ordering.js',
                    array( 'jquery-ui-sortable' )
                );
                wp_enqueue_script( 'prodigy_term_ordering' );

                $taxonomy = isset( $_GET['taxonomy'] ) ? wp_unslash( $_GET['taxonomy'] ) : '';

                $prodigy_term_order_params = array(
                    'taxonomy' => $taxonomy,
                );

                wp_localize_script( 'prodigy_term_ordering', 'prodigy_term_ordering_params',
                    $prodigy_term_order_params );
            }
        }

	}

}
