<?php

/**
 * Prodigy admin product attributes class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Product_Attributes
 */
class Prodigy_Product_Attributes_Box extends Prodigy_Product_Meta_Box {

    /**
     * Prodigy_Product_Attributes constructor.
     */
    public function __construct() {
        parent::__construct();
        add_action( 'add_meta_boxes', [ $this, 'prodigy_attributes_meta_box' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'prodigy_load_attributes_style' ] );
        add_action( 'save_post', [ $this, 'prodigy_save_attributes_meta' ] );
        add_action( 'save_post', [ $this, 'save_default_category' ] );
        add_action( 'pre_post_update', [ $this, 'is_validation' ], 99, 1 );
        add_action( 'wp_ajax_create-attribute-meta-box', [ $this, 'create_attribute_ajax' ] );
        add_action( 'wp_ajax_add-values-meta-box', [ $this, 'add_values_ajax' ] );
        add_action( 'wp_ajax_get-attributes-meta-box', [ $this, 'get_attribute_names_ajax' ] );
        add_action( 'wp_ajax_get-attribute-values-meta-box', [ $this, 'get_attribute_values_ajax' ] );
        add_action( 'wp_ajax_save-attributes-meta-box', [ $this, 'save_attributes_ajax' ] );
    }

    /**
     * Save attributes from ajax
     */
    public function save_attributes_ajax() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );

        $post_id = filter_var($_POST['post_id'], FILTER_VALIDATE_INT);

        if ( isset( $_POST['attribute']) && isset( $_POST['values']) && !empty($_POST['attribute']) && !empty($_POST['values'] ) ) {
            $this->product_attributes->update_attributes_from_product(
                $post_id,
                $_POST['attribute'],
                $_POST['values']
            );
        } else {
            if ( ! empty( $post_id ) ) {

                /*
                 * clear value attributes from aggregation by product
                 * table wp_prodigy_attributes_aggregation
                 */
                $this->product_attributes->delete_aggregation_products_by_attribute( $post_id );
                $this->product_attributes->delete_all_attributes_from_product( $post_id );
            }
        }

        if (is_array($_POST['enable_attributes'])) {
            $enablers = array_diff( $_POST['enable_attributes'], [ '' ] );
            if ( isset( $enablers ) ) {
                update_post_meta( $post_id, '_use_option', serialize( $enablers ) );
            }
        }
    }

    /**
     * Add new values from attribute
     */
    public function add_values_ajax() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
    
        $result     = false;
        $new_values = explode( ",", filter_var( $_POST['attribute_values'] ) );
        $taxonomy   = sanitize_title( $_POST['attribute_name'] );

        register_taxonomy( $taxonomy, null, [] );

        $term_info = get_terms( [
            'taxonomy'   => $taxonomy,
            'hide_empty' => false,
        ] );

        $term_slugs = array_column( $term_info, 'slug' );

        if ( empty( $new_values ) ) {
            echo wp_json_encode( [
                'error'  => 'Enter valid attribute value',
                'status' => 0,
            ] );
            wp_die();
        }

        $list = [];

        $current_attribute = $this->product_attributes->get_attribute_taxonomies_by_name( $taxonomy );

        $new_values = array_unique( $new_values );

        foreach ( $new_values as $value ) {

            $value = filter_var( $value, FILTER_SANITIZE_STRIPPED );

            /**
             * Forbidding existing values
             */
            if ( in_array( trim($value), $term_slugs ) ) {
                continue;
            }

            $result = ! empty( wp_insert_term( $value, $taxonomy, [
                'slug' => sanitize_title( $value )
            ] ) );
            $list[] = $term_current = get_term_by( 'name', $value, $taxonomy );

            /*
             * adding attribute relation to aggregation attribute-products
             */
            $this->product_attributes->set_aggregation_products_by_attributes( $current_attribute[0]->id, $term_current->term_id, $term_current->slug );
        }

        if ( ! $result ) {
            echo wp_json_encode( [
                'error'  => 'Can`t add attribute value or it exists',
                'status' => 0,
            ] );
        } else {
            echo wp_json_encode( [
                'list'   => $list,
                'status' => $result,
            ] );
        }

        wp_die();
    }

    /**
     * Get attribute values for ajax query
     */
    public function get_attribute_values_ajax() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        if ( isset( $_POST['attribute'] ) ) {

            $request_attribute = filter_var($_POST['attribute'], FILTER_SANITIZE_STRING);

            $attribute = explode( ",", $request_attribute );
            echo wp_json_encode( $this->product_attributes->get_values_for_attribute( $attribute[0] ) );
        }
        wp_die();
    }

    /**
     * Create attribute and values for ajax query
     */
    public function create_attribute_ajax() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );

        $array_values = [];
        $new_array_values = [];
        if ( ! empty( $_POST['attribute_values'] ) ) {
            $array_values = explode( ",", $_POST['attribute_values'] );
        }

        if ( empty( $array_values ) ) {
            wp_send_json_error( 'Attribute values are invalid format', 422 );
        }

        foreach ( $array_values as $key => $value ) {
            if ( empty( sanitize_title($value) ) ) {
                continue;
            }
            $new_array_values[] = $value;
        }


        if ( empty( $new_array_values ) ) {
            wp_send_json_error( 'Attribute values are required.', 422 );
        }

        $request_attribute_name = filter_var($_POST['attribute_name'], FILTER_SANITIZE_STRING);
        $request_orderby = filter_var($_POST['orderby'], FILTER_SANITIZE_STRING);

        $response = $this->product_attributes->save(
            $request_attribute_name,
            $request_attribute_name,
            $array_values,
            $request_orderby
        );

        if ( ! $response['result'] ) {
            echo wp_json_encode( [
                'error'  => 'You must create unique attribute.',
                'status' => false,
            ] );
        } else {
            echo wp_json_encode( [
                'attribute' => $response['current_attribute'],
                'status'    => $response['result'],
            ] );
        }

        wp_die();
    }


    /**
     * Create product attributes meta box
     */
    public function prodigy_attributes_meta_box() {
        add_meta_box(
            'prodigy-product-attributes',
            __( 'Product Attributes', 'prodigy' ),
            [ $this, 'prodigy_show_attributes_meta_box' ],
            Prodigy::get_prodigy_product_type(),
            'advanced',
            'low'
        );
    }

    /**
     * Set scripts for meta box
     */
    public function prodigy_load_attributes_style() {
        wp_enqueue_style( 'wp-jquery-ui-dialog' );
        wp_enqueue_script( 'jquery-ui-dialog' );
        wp_enqueue_script( 'select2',
            dirname( plugin_dir_url(__FILE__) ) . '/web/libraries/select2/js/select2.min.js',
            [ 'jquery' ]
        );

        wp_enqueue_style(
            'select2',
            dirname( plugin_dir_url( __FILE__ ) ) . '/web/libraries/select2/css/select2.min.css'
        );
    }

    /**
     * Render view for meta box
     */
    public function prodigy_show_attributes_meta_box( $post ) {
        // Use nonce for verification
        wp_nonce_field( 'prodigy_show_attributes_meta_box', 'prodigy_attributes_nonce' );
        $is_use_option              = get_post_meta( sanitize_text_field($post->ID), '_use_option', true );
        $is_use_option_unserialized = maybe_unserialize( $is_use_option );
        $attributes                 = get_post_meta( sanitize_text_field($post->ID), Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META, true );
        $attributes_unserialized    = maybe_unserialize( $attributes );
        $all_attributes             = $this->product_attributes->get_attributes();
        $all_attributes_values      = $this->product_attributes->get_all_attributes_with_values_list();

        include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-attributes-meta-box.php';
    }

    /**
     * Get attribute list name for ajax
     */
    public function get_attribute_names_ajax() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        echo wp_json_encode( $this->product_attributes->get_attributes() );

        wp_die();
    }

    /**
     * @param int $post_id
     */
    public function is_validation( int $post_id ) {
        $error = false;

        if ( isset( $_POST['prodigy_attributes_nonce'] ) && wp_verify_nonce( $_POST['prodigy_attributes_nonce'], 'prodigy_show_attributes_meta_box' ) ) {
            if ( ! current_user_can( 'edit_post', sanitize_text_field( $post_id ) ) ) {
                Prodigy_Admin_Notification::displayError( __( 'You cannot edit this post.', 'prodigy' ) );
                $error = true;
            }
        }

        if ( $error ) {
            $url = admin_url( 'post.php?post=' . $post_id ) . '&action=edit';
            wp_redirect( $url );
            exit;
        }
    }

    /**
     * @param int $post_id
     *
     * @return bool
     */
    public function save_default_category( int $post_id ) {

        $list_categories = get_the_terms( $post_id, Prodigy::get_prodigy_category_type() );

        if ( ! empty( $list_categories ) ) {
            return true;
        }

        /**
         * set default category in any case
         */
        $prodigy_default_category = (int) get_option( Prodigy::PRODIGY_DEFAULT_CATEGORY_OPTION, 0 );
        if ( ! empty( $prodigy_default_category ) ) {
            wp_set_post_terms( $post_id, $prodigy_default_category, Prodigy::get_prodigy_category_type(), true );
        }
    }

    /**
     * @param array $data
     *
     * @param int $filter
     *
     * @return array
     */
    public function validate_array_string( array $data, $filter = FILTER_SANITIZE_STRING ) {
        $new_data = [];

        foreach ( $data as $key => $value ) {
            $filter_data = filter_var( $value, $filter );
            if ( $filter_data ) {
                $new_data[$key] = $filter_data;
            }
        }

        return $new_data;
    }

    /**
     * @param array $data
     *
     * @param $filter
     *
     * @return array
     */
    public function prepare_array_validate( array $data, $filter ) {
        $new_data = [];

        foreach ( $data as $key => $value ) {
            if ( is_array( $value ) ) {
                $new_data[] = $this->validate_array_string( $value, $filter );
            }
        }

        return $new_data;
    }

    /**
     * Save attributes data
     *
     * @param int $post_id
     */
    public function prodigy_save_attributes_meta( int $post_id ) {

        if ( isset( $_POST['action'] ) && $_POST['action'] === 'editpost' ) {
            $request_attributes = false;
            $request_values     = false;
            $request_use_option = false;

            if ( isset( $_POST['attribute'] ) && is_array( $_POST['attribute'] ) ) {
                $request_attributes = $this->validate_array_string( $_POST['attribute'] );
            }

            if ( isset( $_POST['values'] ) && is_array( $_POST['values'] ) ) {
                $request_values = $this->prepare_array_validate( $_POST['values'], FILTER_VALIDATE_INT );
            }

            if ( isset( $_POST['use_option'] ) && is_array( $_POST['use_option'] ) ) {
                $request_use_option = $this->validate_array_string( $_POST['use_option'] );
            }

            if ( ! empty( $request_attributes ) && ! empty( $request_values ) ) {
                delete_post_meta( $post_id, Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META );
                delete_post_meta( $post_id, '_use_option' );
            }

            if ( isset( $request_attributes) && isset( $request_values) && !empty($request_values ) && !empty($request_attributes ) ) {

                $this->product_attributes->update_attributes_from_product(
                    $post_id,
                    $request_attributes,
                    $request_values,
                    $request_use_option
                );
            } else {
                /*
                 * clear value attributes from aggregation by product
                 * table wp_prodigy_attributes_aggregation
                 */
                $this->remove_relationship_attributes_from_product($post_id);
                update_post_meta( $post_id, Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META, '' );
                $this->product_attributes->delete_aggregation_products_by_attribute( $post_id );
                $this->product_attributes->delete_all_attributes_from_product( $post_id );
            }
            if ( isset( $_POST['use_option'] ) ) {
                update_post_meta( $post_id, '_use_option', serialize( $request_use_option ) );
            } else {
                update_post_meta( $post_id, '_use_option', '' );
            }
        }
    }
    
    /**
     * @param $post_id
     */
    public function remove_relationship_attributes_from_product( $post_id ) {
        $list = maybe_unserialize( get_post_meta( sanitize_text_field( $post_id ), Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META, true ) );
        if ( isset( $list[0] ) && isset( $list[0]['values'] ) && is_array($list[0]['values']) ) {
            foreach ( $list[0]['values'] as $item ) {
                $term_info = get_term( $item );
                
                if ( ! empty( $term_info ) ) {
                    wp_remove_object_terms( $post_id, $term_info->slug, $term_info->taxonomy );
                }
            }
        }
    }

}

new Prodigy_Product_Attributes_Box();
