<?php

/**
 * Prodigy admin product bulk edit class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

defined( 'ABSPATH' ) || exit;

require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-products-bulk-listener.php';

/**
 * Class Prodigy_Product_Bulk_Edit
 */
class Prodigy_Product_Bulk_Edit {
    const INCREASE_PRICE = 1;
    const DECREASE_PRICE = 2;
    const SALE_DECREASE_PRICE_FOR_REGULAR = 3;
    const COUNT_DIGITS_FOR_ROUND = 2;

    const PRICE_MUST_BE_LESS_PERCENTS = 'Must be less than 100%';
    const PRICE_SHOULD_BE_MORE = 'Price should be more 0';
    const SALE_PRICE_MUST_BE_LESS = 'Sale price must be lower than Regular price';

    protected $process_all;


    /**
     * Prodigy_Product_Bulk_Edit constructor.
     */
    public function __construct() {
        add_action( 'bulk_edit_custom_box', [ $this, 'prodigy_bulk_edit' ], 10, 2 );
        add_action( 'pre_post_update', [ $this, 'save_prodigy_bulk_edit_data' ], 99, 2 );

        $this->process_all = new Prodigy_Product_Bulk_Listener();
    }

    /**
     * @param array $form_data
     */
    public function validate_data( $form_data ) {
        $error = false;
        if ( ! empty( $form_data['regular_price_value'] ) ) {
            $value = $form_data['regular_price_value'];
            if ( strripos( $value, "%" ) && $value > 100 ) {
                Prodigy_Admin_Notification::displayError( __( self::PRICE_MUST_BE_LESS_PERCENTS, 'prodigy' ) );
                $error = true;
            }

            $value = str_replace( "%", "", $form_data['regular_price_value'] );

            if ( empty( filter_var( $value, FILTER_VALIDATE_FLOAT ) ) ) {
                Prodigy_Admin_Notification::displayError( __( 'Field Regular price is not valid', 'prodigy' ) );
                $error = true;
            }
        }

        if ( ! empty( $form_data['product_sale_price'] ) ) {
            $value = $form_data['product_sale_price'];
            if ( strripos( $value, "%" ) && $value > 100 ) {
                Prodigy_Admin_Notification::displayError( __( self::PRICE_MUST_BE_LESS_PERCENTS, 'prodigy' ) );
                $error = true;
            }
            $value = str_replace( "%", "", $form_data['product_sale_price'] );
            if ( empty( filter_var( $value, FILTER_VALIDATE_FLOAT ) ) ) {
                Prodigy_Admin_Notification::displayError( __( 'Field Sale price is not valid', 'prodigy' ) );
                $error = true;
            }
        }

        if ( ! empty( $form_data['product_stock_quantity'] ) ) {
            if ( empty( filter_var( $form_data['product_stock_quantity'], FILTER_VALIDATE_INT ) ) ) {
                Prodigy_Admin_Notification::displayError( __( 'Field Stock quantity is not valid', 'prodigy' ) );
                $error = true;
            }
        }

        if ( $error ) {
            $url = admin_url( 'edit.php?post_status=publish&post_type='.Prodigy::get_prodigy_product_type() );
            wp_redirect( $url );
            exit;
        }
    }

    /**
     * Validate data
     */
    public function save_prodigy_bulk_edit_data() {

        if ( isset( $_GET['bulk_edit'] ) ) {

            $this->validate_data( $_GET );

            $post_ids = ( isset( $_GET['post'] ) && ! empty( $_GET['post'] ) ) ? $_GET['post'] : [];

            if ( ! empty( $post_ids ) && is_array( $post_ids ) ) {

                foreach ( $post_ids as $key => $post_id ) {
                    $error['product_update'][ $post_id ] = $this->save_price( $post_id, $_GET );

                    if ( $_GET['product_stock_status'] !== '' ) {
                        update_post_meta(
                            $post_id,
                            'product_stock_status',
                            $_GET['product_stock_status']
                        );

                        $is_hide_product = get_option( 'pg_inventory_hide_products' );
                        if ( isset( $is_hide_product ) &&
                             $_GET['product_stock_status'] === Prodigy::PRODIGY_HIDE_STATUS
                        ) {
                            global $wpdb;

                            $wpdb->get_results(
                                $wpdb->prepare(
                                    "UPDATE $wpdb->posts SET post_status = %s WHERE ID = %d ",
                                    [ Prodigy::PRODIGY_HIDE_STATUS, $post_id ]
                                )
                            );
                        }
                    }

                    if ( $_GET['_status'] == '-1' && !empty( $error['product_update'][ $post_id ] ) ) {
                        /** Add product to queue */
                        $this->process_all->push_to_queue( $post_id );
                        update_post_meta(
                            $post_id,
                            Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                            Prodigy_Product_Synchronize::PENDING_STATUS
                        );
                    }
                }

                /** Dispatch queue */
                $this->process_all->save()->dispatch();
            }

            if ( ! empty( $error['product_update'] ) ) {
                $error = $this->format_error( $error['product_update'] );
                Prodigy_Admin_Notification::displayError( __( $error, 'prodigy' ) );

                if ( $error ) {

                    $url = admin_url( 'edit.php?post_status=publish&post_type='.Prodigy::get_prodigy_product_type() );
                    wp_redirect( $url );
                    exit;
                }
            }
        }
    }

    /**
     * @param string $column_name
     * @param string $post_type
     */
    public function prodigy_bulk_edit( $column_name, $post_type ) {
        if ( $column_name !== 'regular_price' || Prodigy::get_prodigy_product_type() !== $post_type ) {
            return;
        }

        include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-bulk-edit.php';
    }

    /**
     * @param array $errors
     *
     * @return string
     */
    public function format_error( $errors ) {
        $error_message = '';
        foreach ( $errors as $key => $error ) {
            $product = get_post($key);

            if ( ! empty( $error ) ) {
                $error_message .= 'Product ' . $product->post_title . ' - ' . $error . '<br/>';
            }
        }

        return $error_message;
    }

    /**
     * @param int   $products_id
     * @param array $form_data
     *
     * @return string
     */
    private function save_price( $products_id, $form_data ) {
        if ( ! empty( $form_data['regular_price_value'] && ! empty( $form_data['change_regular_price'] ) ) ) {
            $new_regular_price = $this->calculate_product_price(
                $form_data['change_regular_price'],
                get_post_meta( $products_id, 'product_regular_price' ),
                trim( $form_data['regular_price_value'] )
            );
            if ( $new_regular_price < 0 ) {
                $error = self::PRICE_SHOULD_BE_MORE;
            }
        }

        if ( ! empty( $form_data['sale_price_value'] ) && ! empty( $form_data['change_sale_price'] ) && empty( $error ) ) {
            $price = ( self::SALE_DECREASE_PRICE_FOR_REGULAR === $form_data['change_sale_price'] ) ?
                get_post_meta( $products_id, 'product_regular_price' ) :
                get_post_meta( $products_id, 'product_sale_price' );

            $new_sale_price = $this->calculate_product_price(
                $form_data['change_sale_price'],
                $price,
                trim( $form_data['sale_price_value'] )
            );
            if ( $new_sale_price < 0 ) {
                $error = self::PRICE_SHOULD_BE_MORE;
            }
        }

        if (
            ( empty( $form_data['regular_price_value'] ) && empty( $form_data['regular_price_value'] ) ) &&
            ( ! empty( $form_data['sale_price_value'] ) && ! empty( $form_data['change_sale_price'] ) ) &&
            isset( $new_sale_price ) &&
            empty( $error )
        ) {
            $current_regular_price = get_post_meta( $products_id, 'product_regular_price', true );
            if ( $current_regular_price > $new_sale_price ) {
                update_post_meta(
                    $products_id,
                    'product_sale_price',
                    $new_sale_price
                );
            } else {
                $error = self::SALE_PRICE_MUST_BE_LESS;
            }
        }


        if (
            ( empty( $form_data['sale_price_value'] ) && empty( $form_data['change_sale_price'] ) ) &&
            ( ! empty( $form_data['regular_price_value'] ) && ! empty( $form_data['change_regular_price'] ) ) &&
            isset( $new_regular_price ) &&
            empty( $error )
        ) {
            $current_sale_price = get_post_meta( $products_id, 'product_sale_price', true );
            if ( $new_regular_price > $current_sale_price ) {
                update_post_meta(
                    $products_id,
                    'product_regular_price',
                    $new_regular_price
                );
            } else {
                $error = self::SALE_PRICE_MUST_BE_LESS;
            }
        }


        if ( isset( $new_regular_price ) && isset( $new_sale_price ) && empty( $error ) ) {
            if ( $new_regular_price > $new_sale_price ) {
                update_post_meta(
                    $products_id,
                    'product_regular_price',
                    $new_regular_price
                );
                update_post_meta(
                    $products_id,
                    'product_sale_price',
                    $new_sale_price
                );
            } else {
                $error = self::SALE_PRICE_MUST_BE_LESS;
            }
        }

        return $error;
    }

    /**
     * @param string $filter
     * @param float  $product_price
     * @param float  $form_price
     *
     * @return float
     */
    private function calculate_product_price( $filter, $product_price, $form_price ) {
        $price = 0;
        switch ( $filter ) {
            case self::INCREASE_PRICE:
                if ( strripos( $form_price, "%" ) ) {
                    preg_match_all( '!\d+!', $form_price, $matches );
                    $price = $product_price[0] + ( ( $product_price[0] * $matches[0][0] ) / 100 );
                } else {
                    $price = $product_price[0] + $form_price;
                }
                break;
            case self::DECREASE_PRICE || self::SALE_DECREASE_PRICE_FOR_REGULAR:
                if ( strripos( $form_price, "%" ) ) {
                    preg_match_all( '!\d+!', $form_price, $matches );
                    $price = $product_price[0] - ( ( $product_price[0] * $matches[0][0] ) / 100 );
                } else {
                    $price = $product_price[0] - $form_price;
                }
                break;
            default:
                $price = $product_price;
        }

        return round( $price, self::COUNT_DIGITS_FOR_ROUND );
    }

}

new Prodigy_Product_Bulk_Edit();