<?php

/**
 * Prodigy admin product price class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_Price_Box
 */
class Prodigy_Product_Price_Box extends Prodigy_Product_Meta_Box {

    /**
     * Prodigy_Admin_Metaboxes constructor.
     */
    public function __construct() {
        parent::__construct();
        add_action( 'add_meta_boxes', [ $this, 'prodigy_price_meta_box' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'prodigy_load_price_style' ] );
        add_action( 'save_post', [ $this, 'prodigy_save_price_meta' ], 1, 3 );
        add_action( 'save_post', [ $this, 'refresh_permalink_post' ], 1, 3 );
        add_action( 'pre_post_update', [ $this, 'is_validate_price' ], 99, 2 );
    }
    
    /**
     * Refresh permalinks
     */
    public function refresh_permalink_post() {
        if ( get_option( 'prodigy_refresh_permalinks' ) < 3 ) {
            $prodigy_refresh_permalinks = (int)get_option( 'prodigy_refresh_permalinks' );
            Prodigy::refresh_permalink();
            update_option('prodigy_refresh_permalinks', ++$prodigy_refresh_permalinks);
        }
    }

    /**
     * Registration price meta-box
     */
    public function prodigy_price_meta_box() {
        add_meta_box(
            'prodigy-product-price',
            __( 'Product Price', 'prodigy' ),
            [ $this, 'prodigy_show_price_meta_box' ],
            Prodigy::get_prodigy_product_type(),
            'advanced',
            'high'
        );
    }

    /**
     * Registration scripts
     */
    public function prodigy_load_price_style() {
        wp_enqueue_script( 'jquery-ui-datepicker' );
    }

    /**
     * Render meta-box
     *
     * @param $post
     */
    public function prodigy_show_price_meta_box( $post ) {

        // Use nonce for verification
        wp_nonce_field( 'prodigy_show_price_meta_box', 'prodigy_price_nonce' );

        $from_date = get_post_meta( $post->ID, 'product_from_date', true );
        $to_date   = get_post_meta( $post->ID, 'product_to_date', true );

        $regular_price    = get_post_meta( $post->ID, 'product_regular_price', true ) ?? '';
        $sale_price       = get_post_meta( $post->ID, 'product_sale_price', true ) ?? '';
        $check_sale_price = get_post_meta( $post->ID, 'product_check_sale_price', true ) ?? '';

        include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-price-meta-box.php';
    }

    /**
     * @param        $date
     * @param string $format
     *
     * @return bool
     */
    private function validate_date( $date, $format = 'Y-m-d H:i:s' ) {
        $d = DateTime::createFromFormat( $format, $date );

        return $d && $d->format( $format ) == $date;
    }

    /**
     * @param int $post_id
     */
    public function is_validate_price( int $post_id ) {

        $error = false;

        if ( ! isset( $_POST['_inline_edit'] ) ) {
            if ( ! empty( $_POST['regular_price'] ) ) {
                if ( empty( filter_var( $_POST['regular_price'], FILTER_VALIDATE_FLOAT ) ) ) {
                    Prodigy_Admin_Notification::displayError( esc_html__( 'Field price is not valid', 'prodigy' ) );
                    $error = true;
                }
            }

            if ( ! empty( $_POST['check_sale_price'] ) ) {
                if ( empty( filter_var( $_POST['regular_price'],
                        FILTER_VALIDATE_FLOAT ) ) || empty( filter_var( $_POST['sale_price'],
                        FILTER_VALIDATE_FLOAT ) ) ) {
                    Prodigy_Admin_Notification::displayError( esc_html__( 'Invalid Regular price or Sale price', 'prodigy' ) );
                    $error = true;
                }

                if ( abs( $_POST['regular_price'] ) < abs( $_POST['sale_price'] ) ) {
                    Prodigy_Admin_Notification::displayError( esc_html__( 'The regular_price must be higher than the sale_price',
                        'prodigy' ) );
                    $error = true;
                }

                if ( empty( $this->validate_date( $_POST['from_date'],
                        'Y-m-d' ) ) || empty( $this->validate_date( $_POST['to_date'], 'Y-m-d' ) ) ) {
                    Prodigy_Admin_Notification::displayError( esc_html__( 'Fields from_date or to_date is not valid',
                        'prodigy' ) );
                    $error = true;
                }

                if ( strtotime( $_POST['to_date'] ) < strtotime( $_POST['from_date'] ) ) {
                    Prodigy_Admin_Notification::displayError( esc_html__( 'Invalid from_date or to_date',
                        'prodigy' ) );
                    $error = true;
                }
            }
        }

        if ( $error ) {
            $url = admin_url( 'post.php?post=' . $post_id ) . '&action=edit';
            wp_redirect( $url );
            exit;
        }
    }

    /**
     * Save data for meta-box
     *
     * @param $post_id
     *
     * @return mixed
     */
    public function prodigy_save_price_meta( $post_id ) {
        if ( ! isset( $_POST['prodigy_price_nonce'] ) || ! wp_verify_nonce( $_POST['prodigy_price_nonce'],
                'prodigy_show_price_meta_box' ) ) {
            return;
        }

        if ( ! current_user_can( 'edit_post', $post_id ) ) {
            return;
        }

        if ( ! empty( $_POST['from_date'] ) ) {
            update_post_meta( $post_id, 'product_from_date', sanitize_text_field( $_POST['from_date'] ) );
        }

        if ( ! empty( $_POST['to_date'] ) ) {
            update_post_meta( $post_id, 'product_to_date', sanitize_text_field( $_POST['to_date'] ) );
        }

        if ( isset( $_POST['regular_price'] ) ) {
            update_post_meta( $post_id, 'product_regular_price', sanitize_text_field( $_POST['regular_price'] ) );
        }

        if ( isset( $_POST['sale_price'] ) ) {
            update_post_meta( $post_id, 'product_sale_price', sanitize_text_field( $_POST['sale_price'] ) );
        }

        if ( isset( $_POST['check_sale_price'] ) ) {
            update_post_meta( $post_id, 'product_check_sale_price', sanitize_text_field($_POST['check_sale_price']) );
        } else {
            delete_post_meta( $post_id, 'product_check_sale_price' );
        }

        $sort_price = ! empty( $_POST['sale_price'] ) ? $_POST['sale_price'] : $_POST['regular_price'];
        update_post_meta( $post_id, 'product_sort_price', sanitize_text_field( $sort_price ) );
    }

}

new Prodigy_Product_Price_Box();
