<?php

/**
 * Prodigy admin product synchronize class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-aggregate-product-data-synchronize.php';
include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-product.php';

/**
 * Class Prodigy_Product_Synchronize
 */
class Prodigy_Product_Synchronize extends Prodigy_Main_Abstract {

    protected $prodigy_synchronize;
    public $prodigy_product;
    public $prodigy_cart;

    const SUCCESS_STATUS = 'success';
    const PENDING_STATUS = 'pending';
    const FAILED_STATUS = 'failed';

    const PRODUCT_DONT_SYNCHRONIZE_MESSAGE = 'We have put the product into the Unsynchronized status because you don’t have a connection with the Hosted System yet';

    /**
     * Prodigy_Product_Synchronize constructor.
     */
    public function __construct() {
        parent::__construct();

        $this->prodigy_synchronize = new Prodigy_Aggregate_Data_Synchronize();
        $this->prodigy_product = new Prodigy_Product();
        $this->prodigy_cart = new Prodigy_Cart();

        add_action( 'pre_post_update', [ $this, 'synchronize_remote_product' ] );
        add_action( 'before_delete_post', [ $this, 'delete_remote_product' ] );
        add_action( 'save_post', [ $this, 'unsynchronize_product_change_status' ] );
        add_action( 'wp_ajax_retry-synchronize', [ $this, 'synchronize_remote_product' ] );
        add_action( 'wp_trash_post', [ $this, 'move_product_to_trash' ] );
        add_action( 'untrashed_post', [ $this, 'restore_product_from_trash' ] );
    }

    /**
     * @param int $product_id
     */
    public function move_product_to_trash( $product_id ) {
        if (Prodigy_Type::is_prodigy_product_post_type(get_post_type($product_id))) {
            $exist_remote_data = $this->prodigy_synchronize->get_remote_product_data( $product_id );
            if ( isset( $exist_remote_data ) && $exist_remote_data['remote_product_id'] != null ) {
                $response = $this->api_client->hide_product( $exist_remote_data['remote_product_id'] );
                $this->show_trash_message( $response, $product_id );
            }

            $variants = $this->get_children_products( $product_id );

            if ( ! empty( $variants ) ) {
                foreach ( $variants as $item ) {
                    $is_removed_item = $this->prodigy_cart->remove_item_from_local_cart( $item['ID'],
                        $_COOKIE['prodigy_user'] );
                    if ( $is_removed_item ) {
                        update_option( 'prodigy_cart_removed_products', $item['ID'] );
                        update_option( 'prodigy_widget_cart_removed_products', $item['ID'] );
                        wp_update_post( [ 'ID' => $item['ID'], 'post_status' => 'trash' ] );
                    }
                }
            } else {
                $is_removed_item = $this->prodigy_cart->remove_item_from_local_cart( $product_id,
                    $_COOKIE['prodigy_user'] );
                if ( $is_removed_item ) {
                    update_option( 'prodigy_cart_removed_products', $product_id );
                    update_option( 'prodigy_widget_cart_removed_products', $product_id );
                }
            }

            $order_token = $this->prodigy_cart->get_token_for_current_order( $_COOKIE['prodigy_user'] );
            if ( ! empty( $order_token ) ) {
                $this->api_client->delete_order_item( $order_token );
            } else {
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger', 'Delete order item -  Order token not found', 'error' );
                }
            }
        }
    }

    /**
     * @param $product_id
     *
     * @return array|object|null
     */
    public function get_children_products( $product_id ) {
        global $wpdb;
        return $item_product = $wpdb->get_results(
            $wpdb->prepare(
                "select * from {$wpdb->posts} where post_parent = %d",
                $product_id
            ),
            ARRAY_A
        );
    }

    /**
     * @param int $product_id
     */
    public function restore_product_from_trash( $product_id ) {

        if (Prodigy_Type::is_prodigy_product_post_type(get_post_type($product_id))) {

            $exist_remote_data = $this->prodigy_synchronize->get_remote_product_data( $product_id );
            if ( isset( $exist_remote_data ) && $exist_remote_data['remote_product_id'] != null ) {
                $response = $this->api_client->restore_product( $exist_remote_data['remote_product_id'] );
                $this->show_trash_message( $response, $product_id );
            }

            $variants = $this->get_children_products( $product_id );

            if ( ! empty( $variants ) ) {
                foreach ( $variants as $item ) {
                    wp_update_post( [ 'ID' => $item['ID'], 'post_status' => 'publish' ] );
                }
            }
        }
    }

    /**
     * @param array $response
     * @param int $product_id
     */
    public function show_trash_message( $response, $product_id ) {

        if (Prodigy_Type::is_prodigy_product_post_type(get_post_type($product_id))) {

            $data_response = json_decode( wp_remote_retrieve_body( $response ), 1 );
            $status_code   = wp_remote_retrieve_response_code( $response );
            $message       = '';
            if ( $status_code == 401 ) {
                $message = esc_html__( self::PRODUCT_DONT_SYNCHRONIZE_MESSAGE, 'prodigy' );
            }

            if ( ! empty( $data_response ) ) {
                if ( $status_code == 404 || ( array_key_exists( 'error', $data_response ) ) ) {
                    $message = esc_html__( ' Not found product ! ', 'prodigy' );
                }
            }

            if ( ! empty( $message ) ) {
                Prodigy_Admin_Notification::displayError( $message );
                $_POST['unsychronize_product'] = true;
                $this->prodigy_synchronize->update_product_status( $product_id,
                    Prodigy::PRODIGY_UNSYNCHRONIZED_POST_STATUS );
                update_post_meta(
                    $product_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            }
        }
    }

    /**
     * @param int $post_id
     */
    public function unsynchronize_product_change_status( $post_id ) {
        if (
            Prodigy_Type::is_prodigy_product_post_type(get_post_type($post_id)) &&
            isset($_POST['unsychronize_product']) &&
            $_POST['unsychronize_product'] == true
        ) {
            remove_action( 'save_post', [ $this, 'unsynchronize_product_change_status' ] );
            $this->prodigy_synchronize->update_product_status( $post_id, Prodigy::PRODIGY_UNSYNCHRONIZED_POST_STATUS );
            unset( $_POST['unsychronize_product'] );
            add_action( 'save_post', [ $this, 'unsynchronize_product_change_status' ] );
        }
    }

    /**
     * @param int $post_id
     *
     * @return array
     */
    public function synchronize_remote_product( $post_id ) {
//        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        if (Prodigy_Type::is_prodigy_product_post_type(get_post_type($post_id))) {
            if ( ( $_POST['action'] === 'editpost' || $_POST['action'] === Prodigy_Aggregate_Data_Synchronize::RETRY_SYNCHRONIZE_ACTION ) ) {

                $postId                = ! empty( $post_id ) ? $post_id : sanitize_text_field( $_POST['post_id'] );
                $show_product_response = [];

                $exist_remote_data = $this->prodigy_synchronize->get_remote_product_data( $postId );
                if ( isset( $exist_remote_data ) && $exist_remote_data['remote_product_id'] != null ) {
                    $show_product_response = $this->api_client->show_product( $exist_remote_data['remote_product_id'] );
                }

                if ( $_POST['action'] === Prodigy_Aggregate_Data_Synchronize::RETRY_SYNCHRONIZE_ACTION ) {
                    $form_params = $this->prodigy_synchronize->prepare_request_for_retry_product( $postId );
                    $response    = $this->api_client->create_product( json_encode( $form_params ) );

                    if ( isset( $postId ) ) {
                        $this->update_meta_data_for_retry( $postId, $response, 201 );
                    }
                } else {
                    $form_params = $this->prodigy_synchronize->prepare_request( $postId, $_POST );
                }

                if ( empty( $show_product_response ) || array_key_exists( 'error', $show_product_response ) ) {
                    $response = $this->api_client->create_product( json_encode( $form_params ) );

                    if ( isset( $postId ) ) {
                        $this->update_meta_data( $postId, $response, 201 );
                    }

                } else {
                    $product_id = $exist_remote_data['remote_product_id'];
                    if ( ! empty( $form_params['variants'] || ! empty( $exist_remote_data['variants'] ) ) ) {
                        unset( $exist_remote_data['remote_product_id'] );
                        $form_params['variants'] = $this->prodigy_synchronize->update_variants_for_remote_product(
                            $form_params['variants'],
                            $exist_remote_data['variants']
                        );
                    }

                    $response = $this->api_client->update_product( $product_id, json_encode( $form_params ) );
                    $this->update_meta_data( $postId, $response, 200 );
                }

                return $_POST;
            }
        }
    }

    /**
     * @param int $post_id
     */
    public function delete_remote_product( $post_id ) {
        if (Prodigy_Type::is_prodigy_product_post_type(get_post_type($post_id))) {
            $exist_remote_data = $this->prodigy_synchronize->get_remote_product_data( $post_id );
            $meta_status       = get_post_meta( $post_id,
                Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE );

            if ( ! empty( $exist_remote_data ) && $exist_remote_data['remote_product_id'] && $meta_status[0] === 'success' ) {
                $response      = $this->api_client->delete_product( $exist_remote_data['remote_product_id'] );
                $status_code   = wp_remote_retrieve_response_code( $response );
                $data_response = json_decode( wp_remote_retrieve_body( $response ), 1 );

                if ( $status_code == 204 || $status_code == 200 || $status_code == 201 ) {
                    $this->prodigy_synchronize->delete_remote_product_data( $post_id );

                    $variants = Prodigy_Product_Variants::get_product_variants( $post_id );
                    foreach ( $variants as $variant ) {
                        wp_delete_post( $variant['ID'] );
                    }

                } else {
                    /** @var $error - set errors to log */
                    if (is_array($data_response) && isset($data_response['errors']) && is_array($data_response['errors'])) {
                        foreach ( $data_response['errors'] as $error ) {
        
                            $error['post_id'] = $post_id;
                            if ( PRODIGY_DEBUG_MODE ) {
                                do_action( 'logger', $error, 'error' );
                            }
                        }
                    }
                    
                    Prodigy_Admin_Notification::displayError( esc_html__( ' Product is not synchronized ! ',
                        'prodigy' ) );
                    $url = admin_url( 'edit.php?post_status=trash&post_type=' . Prodigy::get_prodigy_product_type() );
                    wp_redirect( $url );
                    exit;
                }

                return;
            }
        }
    }

    /**
     * @param int $post_id
     * @param array $response
     * @param string $success_status
     */
    public function update_meta_data_for_retry( $post_id, $response, $success_status ) {
        if (Prodigy_Type::is_prodigy_product_post_type(get_post_type($post_id))) {
            $status_code = wp_remote_retrieve_response_code( $response );

            $data_response = json_decode( wp_remote_retrieve_body( $response ), 1 );

            $error_message = '';
            if ( $status_code == 401 || $status_code == 500 ) {
                $error_message = esc_html__( self::PRODUCT_DONT_SYNCHRONIZE_MESSAGE, 'prodigy' );
                update_post_meta(
                    $post_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            }

            if ( ! empty( $data_response ) &&
                 ( array_key_exists( 'error', $data_response ) || $status_code != $success_status )
            ) {

                /** @var $error - set errors to log */
                foreach ( $data_response['errors'] as $error ) {
                    $error['post_id'] = $post_id;
                    if ( PRODIGY_DEBUG_MODE ) {
                        do_action( 'logger', $error, 'error' );
                    }
                }

                $error_message = esc_html__( ' Product is not synchronized ! ', 'prodigy' );
                update_post_meta(
                    $post_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            } elseif ( empty( $data_response ) ) {

                $error_message = __( 'We were unable to synchronize the product due to an unexpected error.  Please try again later.',
                    'prodigy' );
                update_post_meta(
                    $post_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            } else {
                if ( empty( $error_message ) ) {
                    $remote_data = $this->prodigy_synchronize->parse_variants_from_response( json_decode( wp_remote_retrieve_body( $response ),
                        1 ) );
                    $this->prodigy_synchronize->update_remote_product_data( $post_id, $remote_data );
                    update_post_meta(
                        $post_id,
                        Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                        Prodigy_Product_Synchronize::SUCCESS_STATUS
                    );
                    $status = get_post_status( $post_id );
                    if ( $status === Prodigy::PRODIGY_UNSYNCHRONIZED_POST_STATUS ) {
                        $this->prodigy_synchronize->update_product_status( $post_id, 'publish' );
                        echo json_decode( [ 'status' => Prodigy_Product_Synchronize::SUCCESS_STATUS ] );
                        die();
                    }
                }
            }

            if ( ! empty( $error_message ) ) {
                Prodigy_Admin_Notification::displayError( $error_message );
                $_POST['unsychronize_product'] = true;
                echo wp_json_encode( [ 'status' => Prodigy_Product_Synchronize::FAILED_STATUS ] );
                die();
            }
        }
    }

    /**
     * @param int $post_id
     * @param array $response
     * @param string $success_status
     */
    public function update_meta_data( $post_id, $response, $success_status ) {
        if (Prodigy_Type::is_prodigy_product_post_type(get_post_type($post_id))) {
            $status_code = wp_remote_retrieve_response_code( $response );

            $data_response = json_decode( wp_remote_retrieve_body( $response ), 1 );

            $message = '';


            if (
                ! empty( $data_response ) &&
                ( array_key_exists( 'error', $data_response ) || $status_code != $success_status )
            ) {
                /** @var $error - set errors to log */
                foreach ( $data_response['errors'] as $error ) {
                    $error['post_id'] = $post_id;
                    if ( PRODIGY_DEBUG_MODE ) {
                        do_action( 'logger', $error, 'error' );
                    }
                }
                $message = esc_html__( 'Product is not synchronized !', 'prodigy' );
                update_post_meta(
                    $post_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            } elseif ( $status_code == 401 || $status_code == 500 ) {
                unset( $data_response['traces'] );
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger', $data_response, 'error' );
                }
                $message = esc_html__( self::PRODUCT_DONT_SYNCHRONIZE_MESSAGE, 'prodigy' );
                update_post_meta(
                    $post_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            } elseif ( empty( $data_response ) ) {

                $message = __( 'We were unable to synchronize the product due to an unexpected error.  Please try again later.',
                    'prodigy' );
                update_post_meta(
                    $post_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            } else {
                $remote_data = $this->prodigy_synchronize->parse_variants_from_response( json_decode( wp_remote_retrieve_body( $response ),
                    1 ) );
                $this->prodigy_synchronize->update_remote_product_data( $post_id, $remote_data );
                update_post_meta(
                    $post_id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::SUCCESS_STATUS
                );
                $status = get_post_status( $post_id );
                if ( $status === Prodigy::PRODIGY_UNSYNCHRONIZED_POST_STATUS ) {
                    $this->prodigy_synchronize->update_product_status( $post_id, 'publish' );
                }
            }

            if ( ! empty( $message ) ) {
                Prodigy_Admin_Notification::displayError( $message );
                $_POST['unsychronize_product'] = true;
            }
        }
    }
}

new Prodigy_Product_Synchronize();