<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

add_filter( 'terms_clauses', 'terms_clauses_prodigy', 99, 3 );

/**
 * Add term ordering to get_terms.
 *
 * @param array $clauses    Clauses.
 * @param array $taxonomies Taxonomies.
 * @param array $args       Arguments.
 * @return array
 */
function terms_clauses_prodigy( $clauses, $taxonomies, $args ) {
    global $wpdb;

    if ( isset($taxonomies[0]) && $taxonomies[0] === Prodigy::get_prodigy_category_type()) {

        // No sorting when menu_order is false.
        if (isset($args['menu_order']) && (false === $args['menu_order'] || 'false' === $args['menu_order'])) {
            return $clauses;
        }

        // No sorting when orderby is non default.
        if (isset($args['orderby']) && 'name' !== $args['orderby']) {
            return $clauses;
        }

        // No sorting in admin when sorting by a column.
        if (is_admin() && isset($_GET['orderby'])) {
            return $clauses;
        }

        // No need to filter counts.
        if (strpos('COUNT(*)', $clauses['fields']) !== false) {
            return $clauses;
        }

        $clauses['fields'] = $clauses['fields'] . ', tm.meta_value';
        $clauses['join'] .= " LEFT JOIN {$wpdb->termmeta} AS tm ON (t.term_id = tm.term_id AND tm.meta_key = '" .
            esc_sql('order') . "') ";

        if (!isset($args['menu_order']) || !in_array(strtoupper($args['menu_order']), array('ASC', 'DESC'), true)) {
            $args['menu_order'] = 'ASC';
        }

        $order = 'ORDER BY tm.meta_value+0 ' . $args['menu_order'];

        if ($clauses['orderby']) {
            $clauses['orderby'] = str_replace('ORDER BY', $order . ',', $clauses['orderby']);
        } else {
            $clauses['orderby'] = $order;
        }

        // Grouping.
        if (strstr($clauses['fields'], 'tr.object_id')) {
            $clauses['orderby'] = ' GROUP BY t.term_id, tr.object_id ' . $clauses['orderby'];
        } else {
            $clauses['orderby'] = ' GROUP BY t.term_id ' . $clauses['orderby'];
        }
    }

    return $clauses;
}

/**
 * Move a term before the a given element of its hierarchy level.
 *
 * @param int    $the_term Term ID.
 * @param int    $next_id  The id of the next sibling element in save hierarchy level.
 * @param string $taxonomy Taxonomy.
 * @param int    $index    Term index (default: 0).
 * @param mixed  $terms    List of terms. (default: null).
 *
 * @return int
 */
function prodigy_reorder_terms( $the_term, $next_id, $taxonomy, $index = 0, $terms = null ) {
    if ( !$terms ) {
        $terms = get_terms( $taxonomy, 'menu_order=ASC&hide_empty=0&parent=0' );
    }
    if ( empty( $terms ) ) {
        return $index;
    }

    $id = intval( $the_term->term_id );

    $term_in_level = false; // Flag: is our term to order in this level of terms.

    foreach ( $terms as $term ) {
        $term_id = intval( $term->term_id );

        if ( $term_id === $id ) { // Our term to order, we skip.
            $term_in_level = true;
            continue; // Our term to order, we skip.
        }
        // the nextid of our term to order, lets move our term here.
        if ( null !== $next_id && $term_id === $next_id ) {
            $index++;
            $index = prodigy_set_term_order( $id, $index, $taxonomy, true );
        }

        // Set order.
        $index++;
        $index = prodigy_set_term_order( $term_id, $index, $taxonomy );

        // If that term has children we walk through them.
        $children = get_terms( $taxonomy, "parent={$term_id}&menu_order=ASC&hide_empty=0" );
        if ( !empty( $children ) ) {
            $index = prodigy_reorder_terms( $the_term, $next_id, $taxonomy, $index, $children );
        }
    }

    // No nextid meaning our term is in last position.
    if ( $term_in_level && null === $next_id ) {
        $index = prodigy_set_term_order( $id, $index + 1, $taxonomy, true );
    }

    return $index;
}

/**
 * Returns true when the passed taxonomy name is a product attribute.
 *
 * @param $name
 *
 * @return bool
 */
function prodigy_taxonomy_is_product_attribute( $name ) {
    global $prodigy_product_attributes;
    $prodigy_product_attributes = array();

    return taxonomy_exists( $name ) && array_key_exists( $name, (array) $prodigy_product_attributes );
}

/**
 * Set the sort order of a term.
 *
 * @param int    $term_id   Term ID.
 * @param int    $index     Index.
 * @param string $taxonomy  Taxonomy.
 * @param bool   $recursive Recursive (default: false).
 * @return int
 */
function prodigy_set_term_order( $term_id, $index, $taxonomy, $recursive = false ) {

    $term_id = (int) $term_id;
    $index   = (int) $index;

    // Meta name.
    if ( prodigy_taxonomy_is_product_attribute( $taxonomy ) ) {
        $meta_name = 'order_' . esc_attr( $taxonomy );
    } else {
        $meta_name = 'order';
    }

    prodigy_update_term_meta( $term_id, $meta_name, $index );

    if ( ! $recursive ) {
        return $index;
    }

    $children = get_terms( $taxonomy, "parent=$term_id&menu_order=ASC&hide_empty=0" );

    foreach ( $children as $term ) {
        $index++;
        $index = prodigy_set_term_order( $term->term_id, $index, $taxonomy, true );
    }

    clean_term_cache( $term_id, $taxonomy );

    return $index;
}

/**
 * Term Meta
 *
 * tables for storing term meta are @deprecated from WordPress 4.4 since 4.4 has its own table.
 *
 * @param int    $term_id    Term ID.
 * @param string $meta_key   Meta key.
 * @param mixed  $meta_value Meta value.
 * @param string $prev_value Previous value. (default: '').
 * @return bool
 */
function prodigy_update_term_meta( $term_id, $meta_key, $meta_value, $prev_value = '' ) {
    return function_exists( 'update_term_meta' ) ? update_term_meta( $term_id, $meta_key, $meta_value, $prev_value ) : update_metadata( 'prodigy_term', $term_id, $meta_key, $meta_value, $prev_value );
}
