<?php

/**
 * Hosted system API client
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


/**
 * Class Prodigy_Api_Client
 */
class Prodigy_Api_Client {
    
    const API_PROTOCOL = PRODIGY_PROTOCOL_DOMAIN;
    const API_CONNECTION_URL = '/api/v1/plugin/connection';
    const PRODUCTS_URL = '/api/v1/plugin/products';
    const PRODUCT_BATCH_URL = '/api/v1/plugin/products/batches/';
    const CATEGORY_URL = '/api/v1/plugin/categories';
    const ORDER_URL = '/api/v1/plugin/orders/';
    const CART_URL = '/api/v1/plugin/cart';
    const LINE_ITEM_URL = '/api/v1/plugin/cart/line_items/';
    const HIDE_ITEM_URL = '/api/v1/plugin/products/%/hiding';
    const UPDATE_CONNECTION_URL = '/api/v1/plugin/connection';
    const GET_INFO_URL = '/api/v1/public/stores/';
    const BATCHES_CATEGORY_URL = '/api/v1/plugin/categories/batches';
    const GET_SOCIAL_ACCOUNTS_URL = '/api/v1/plugin/statistic_apps/google_analytics';
    const DELETE_CART_ITEM_URL = '/api/v1/stores/%store_subdomain%/orders/deleted_item_warning';

    private $request;
    
    /**
     * Prodigy_Api_Client constructor.
     *
     * @param array $headers
     * @param bool $blocking
     * @param int $timeout
     * @param bool $sslverify
     */
    public function __construct(
        array $headers = [ 'Content-Type' => 'application/json' ],
        bool $blocking = true,
        int $timeout = 10,
        bool $sslverify = false
    ) {
        $headers['x-Authorization'] = get_option( 'pg_store_key' );
        $this->request              = [
            'headers'   => $headers,
            'blocking'  => $blocking,
            'timeout'   => $timeout,
            'sslverify' => $sslverify,
        ];
    }
    
    /**
     *
     * @param $response_remote
     *
     * @return array
     */
    public function get_list_errors( $response_remote ) {
        $message_error = json_decode( wp_remote_retrieve_body( $response_remote ) );
        
        if ( is_array( $message_error->errors ) ) {
            return array_column( $message_error->errors, 'detail' );
        }
        
        return [];
    }
    
    /**
     * @param WP_Error $response
     *
     * @return array
     */
    public function response_error( WP_Error $response ) {
        
        return [
            'success' => false,
            'error'   => true,
            'message' => $response->get_error_message(),
            'code'    => $response->get_error_code()
        ];
        
    }
    
    /**
     * @param string $api_key
     * @param string $url
     *
     * @return array|WP_Error
     */
    function setup_connection( string $api_key, string $url ) {
        $this->request['body'] = wp_json_encode( [
            "api_key" => $api_key,
            "url"     => $url
        ] );
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::API_CONNECTION_URL;
        $response = wp_remote_post( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * @param bool $pg_store_key
     *
     * @return array|WP_Error
     */
    public function disconnection( $pg_store_key = false ) {
        $this->request['method'] = 'DELETE';
        if ( ! empty( $pg_store_key ) ) {
            $this->request['headers']['x-Authorization'] = $pg_store_key;
        }
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::API_CONNECTION_URL;
        $response = wp_remote_post( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * @param bool $pg_store_key
     *
     * @param string $store
     *
     * @return array|WP_Error
     */
    public function get_store(  string $store, $pg_store_key = false ) {
        $this->request['method'] = 'GET';
        if ( ! empty( $pg_store_key ) ) {
            $this->request['headers']['x-Authorization'] = $pg_store_key;
        }
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::API_CONNECTION_URL;
        $response = wp_remote_post( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error get info from hosted system' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * ONLY UNINSTALL
     *
     * @param bool $pg_store_key
     *
     * @return array|bool
     */
    public function disconnection_uninstall( $pg_store_key = false ) {
        $this->request['method'] = 'DELETE';
        if ( ! empty( $pg_store_key ) ) {
            $this->request['headers']['x-Authorization'] = $pg_store_key;
        }
        
        $api_url  = self::API_PROTOCOL . PRODIGY_MAIN_DOMAIN . self::API_CONNECTION_URL;
        $response = wp_remote_post( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return true;
    }
    
    /**
     * check on first created product on hosted system
     */
    public function is_first_connected_product() {
        if ( empty( get_option( Prodigy::PRODIGY_FIRST_CONNECTED_PRODUCT ) ) && ! empty( get_option( 'pg_store_key' ) ) ) {
            update_option( Prodigy::PRODIGY_FIRST_CONNECTED_PRODUCT, true );
        }
    }

    /**
     * @param string $url
     *
     * @return array|WP_Error
     */
    public function update_connection_url( $url ) {

        $this->request['body']   = wp_json_encode( [ 'url' => $url ]);
        $this->request['method'] = 'PUT';

        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::UPDATE_CONNECTION_URL;
        $response = wp_remote_request( $api_url, $this->request );

        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }

        return $response;
    }
    
    /**
     * Post data from the Hosted System
     *
     * @param string $params
     *
     * @return array|WP_Error
     */
    function create_product( $params ) {
        
        $this->is_first_connected_product();

        $this->request['body']    = $params;
        $this->request['timeout'] = 25;
        $api_url                  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::PRODUCTS_URL;
        $response                 = wp_remote_post( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }
        
        return $result;
    }
    
    /**
     * @param int $product_id
     * @param string $params
     *
     * @return array|WP_Error
     */
    public function update_product( int $product_id, $params ) {
        $this->request['body']    = $params;
        $this->request['method']  = 'PATCH';
        $this->request['timeout'] = 25;

        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::PRODUCTS_URL . '/' . $product_id;
        $response = wp_remote_request( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }
        
        return $result;
    }


    /**
     * @param array  $params
     * @param string $action_type
     *
     * @return array|WP_Error
     */
    public function create_batch_products( array $params, string $action_type = 'create' ) {

        $params['action_type'] = $action_type;
        $this->request['body']    = json_encode( $params );
        $this->request['timeout'] = 25;
        $api_url                  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::PRODUCT_BATCH_URL;
        $response                 = wp_remote_post( $api_url, $this->request );

        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }

        return $result;
    }

    /**
     * @param int $batch_id
     *
     * @return array|WP_Error
     */
    public function show_batch_products($batch_id ) {
        $this->request['timeout'] = 25;

        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::PRODUCT_BATCH_URL . '/' . $batch_id;
        $response = wp_remote_get( $api_url, $this->request );

        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }

        return $result;
    }


    /**
     * @param int $batch_id
     *
     * @return array|WP_Error
     */
    public function show_batch_products_result($batch_id ) {
        $this->request['timeout'] = 25;

        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::PRODUCT_BATCH_URL . '/' . $batch_id . '/results';
        $response = wp_remote_get( $api_url, $this->request );

        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }

        return $result;
    }
    
    /**
     * @param int $product_id
     *
     * @return array|WP_Error
     */
    public function delete_product( int $product_id ) {
        $this->request['method'] = 'DELETE';
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::PRODUCTS_URL . '/' . $product_id;
        $response = wp_remote_request( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * @param int $product_id
     *
     * @return array|WP_Error
     */
    public function show_product( int $product_id ) {
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::PRODUCTS_URL . '/' . $product_id;
        $response = wp_remote_get( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }
        
        return $result;
    }
    
    
    /**
     * @param int $product_id
     *
     * @return array|WP_Error
     */
    function hide_product( $product_id ) {
        $hide_url = str_replace( "%", $product_id, self::HIDE_ITEM_URL );
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . $hide_url;
        $response = wp_remote_post( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }
        
        return $result;
    }
    
    /**
     * @param int $product_id
     *
     * @return array|WP_Error
     */
    function restore_product( $product_id ) {
        $this->request['method'] = 'DELETE';
        $restore_url             = str_replace( "%", $product_id, self::HIDE_ITEM_URL );
        $api_url                 = self::API_PROTOCOL . PRODIGY_API_DOMAIN . $restore_url;
        $response                = wp_remote_request( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            $result = $this->response_error( $response );
        } else {
            $result = $response;
        }
        
        return $result;
    }
    
    /**
     * @param string $name
     * @param int $parent_id
     *
     * @return array|WP_Error
     */
    function create_category( string $name, int $parent_id = 0 ) {
        
        $slug = sanitize_title( Prodigy::PRODIGY_DEFAULT_NAME_CATEGORY );
        
        if ( $name == $slug ) {
            $this->is_first_connected_product();
        }
        
        $body = [ "name" => $name ];
        if ( ! empty( $parent_id ) ) {
            $body['parent_id'] = $parent_id;
        }
        
        $this->request['body'] = wp_json_encode( $body );
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::CATEGORY_URL;
        $response = wp_remote_post( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * Update product request from the Hosted System
     *
     * @param string $name
     * @param int $category_id
     * @param int $parent_id
     *
     * @return array|WP_Error
     */
    public function update_category( string $name, int $category_id, int $parent_id = 0 ) {
        $body = [ "name" => $name ];
        if ( ! empty( $parent_id ) ) {
            $body['parent_id'] = $parent_id;
        }
        
        $this->request['body']   = wp_json_encode( $body );
        $this->request['method'] = 'PATCH';
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::CATEGORY_URL . '/' . $category_id;
        $response = wp_remote_request( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * @param int $category_id
     *
     * @return array|WP_Error
     */
    public function delete_category( int $category_id ) {
        $this->request['method'] = 'DELETE';
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::CATEGORY_URL . '/' . $category_id;
        $response = wp_remote_request( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * Create batches categories
     *
     * @param array $categories
     *
     * @param string $action Valid options create/update
     *
     * @return array|WP_Error
     */
    public function create_batches_category( array $categories, string $action = 'create' ) {
        $this->request['method'] = 'POST';
    
        $request_body = ['action_type' => $action, 'categories' => $categories];
        $this->request['body'] = wp_json_encode( $request_body );
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::BATCHES_CATEGORY_URL;
        $response = wp_remote_request( $api_url, $this->request );
        
        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $response;
    }
    
    /**
     * @param string $order_token
     *
     * @return array|WP_Error
     */
    public function check_order( $order_token ) {
        $api_url = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::ORDER_URL . $order_token . '?include=line-items';
        unset( $this->request['headers']['Content-Type'] );
        unset( $this->request['method'] );
        unset( $this->request['body'] );
        
        $response = wp_remote_get( $api_url, $this->request );
        
        if ( ! is_wp_error( $response ) ) {
            $result = json_decode( wp_remote_retrieve_body( $response ), true );
        } else {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $result;
    }

    /**
     * @param string $order_token
     *
     * @return bool
     */
    public function delete_order( $order_token ) {
        unset( $this->request['headers']['Content-Type'] );
        $this->request['method'] = 'DELETE';

        $api_url = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::ORDER_URL . $order_token;
        $response    = wp_remote_request( $api_url, $this->request );

        if ( ! is_wp_error( $response ) ) {
            $result = json_decode( wp_remote_retrieve_body( $response ), true );
        } else {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }

        return $result;
    }


    /**
     * @param array $params
     *
     * @return array|WP_Error
     */
    public function set_cart( array $params ) {
        $this->request['body']   = wp_json_encode( $params );
        $this->request['method'] = 'PATCH';
        
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::CART_URL;
        $response = wp_remote_request( $api_url, $this->request );
        
        if ( ! is_wp_error( $response ) ) {
            $result = json_decode( wp_remote_retrieve_body( $response ), true );
        } else {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        }
        
        return $result;
    }
    
    /**
     * @param int $id
     * @param array $params
     *
     * @return bool
     */
    public function update_line_items( int $id, array $params = [] ) {
        $this->request['body']   = wp_json_encode( $params );
        $this->request['method'] = 'PATCH';

        $api_url     = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::LINE_ITEM_URL . $id;
        $response    = wp_remote_request( $api_url, $this->request );
        $status_code = wp_remote_retrieve_response_code( $response );
        
        return ( $status_code === 204 );
    }

    /**
     * @param int $id
     * @param string $order_token
     *
     * @return bool
     */
    public function delete_line_item( int $id, $order_token ) {
        $this->request['method'] = 'DELETE';
        unset( $this->request['headers']['Content-Type'] );
        
        $api_url     = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::LINE_ITEM_URL . $id . '?order_token=' . $order_token;
        $response    = wp_remote_request( $api_url, $this->request );
        $status_code = wp_remote_retrieve_response_code( $response );
        
        return ( $status_code === 204 );
    }

    /**
     * Get google analytics key
     *
     * @return array|WP_Error
     */
    public function get_social_account_key() {
        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . self::GET_SOCIAL_ACCOUNTS_URL;
        do_action( 'logger', $api_url, 'analytic url' );
        $response = wp_remote_get( $api_url, $this->request );

        if ( is_wp_error( $response ) ) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }
            return $this->response_error( $response );
        } else {
            $result = $response;
        }

        return $result;
    }

    /**
     * @param string $order_token
     *
     * @return bool
     */
    public function delete_order_item( $order_token ) {
        unset( $this->request['headers']['Content-Type'] );
        $this->request['method'] = 'DELETE';

        $store_domain = get_option('pg_url_domain_hosted_system');

        $url = '';
        if (!empty($store_domain)) {
            $url = str_replace( "%store_subdomain%", $store_domain, self::DELETE_CART_ITEM_URL );
        }

        $api_url  = self::API_PROTOCOL . PRODIGY_API_DOMAIN . $url . '?order_token=' . $order_token;
        $response = wp_remote_request( $api_url, $this->request );

        if ( ! is_wp_error( $response ) ) {
            $result = json_decode( wp_remote_retrieve_body( $response ), true );
        } else {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response, 'error' );
            }

            return $this->response_error( $response );
        }

        return $result;
    }
}