<?php

/**
 * Prodigy cart class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Cart
 */
class Prodigy_Cart {
    
    const PRODIGY_ORDER_COMPLETED = 1;
    const PRODIGY_ORDER_IN_PROGRESS = 0;

    const CART_ITEMS_TABLE  = 'prodigy_cart_items';
    const REMOTE_DATA_TABLE = 'prodigy_remote_cart_items';
    const ORDER_DATA_TABLE  = 'prodigy_orders';


    public $local_data_table;
    public $remote_data_table;
    public $order_data_table;


    /**
     * Prodigy_Base constructor.
     */
    public function __construct() {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->local_data_table  = $this->wpdb->prefix . self::CART_ITEMS_TABLE;
        $this->remote_data_table = $this->wpdb->prefix . self::REMOTE_DATA_TABLE;
        $this->order_data_table  = $this->wpdb->prefix . self::ORDER_DATA_TABLE;
    }

    /**
     * @param string $current_session
     */
    public function set_user_session( $current_session ) {
        $setting_time = ! empty( get_option( 'pg_custom_expiration_time' ) ) ? get_option( 'pg_custom_expiration_time' ) : get_option( 'pg_cart_expiration_time' );
        $expiry       = strtotime( '+' . $setting_time . ' hours' );
        
        if ( empty( $current_session ) ) {
            @setcookie( 'prodigy_user', generate_uuid(), $expiry, COOKIEPATH, COOKIE_DOMAIN );
        } else {
            @setcookie( 'prodigy_user', $current_session, $expiry, COOKIEPATH, COOKIE_DOMAIN );
            $this->update_expiration_date( $current_session, $expiry );
        }
    }
    
    /**
     * Generate unique hash
     *
     * @return string
     */
    public function generate_uuid() {
        return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),
            mt_rand( 0, 0xffff ),
            mt_rand( 0, 0x0C2f ) | 0x4000,
            mt_rand( 0, 0x3fff ) | 0x8000,
            mt_rand( 0, 0x2Aff ), mt_rand( 0, 0xffD3 ), mt_rand( 0, 0xff4B )
        );
    }

    /**
     * @param $item
     * @param $user_hash
     * @param $order_token
     * @param $api_response
     *
     * @return bool
     */
    public function add_item_to_cart( $item, $user_hash, $order_token, $api_response ) {
        $remote_product_id = filter_var($item['remote_product_id'], FILTER_VALIDATE_INT);
        $local_product_id = filter_var($item['local_product_id'], FILTER_VALIDATE_INT);
        $count = filter_var($item['count'], FILTER_VALIDATE_INT);
        
        $item_product = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "select * from {$this->local_data_table} where hash = %s and remote_product_id = %d",
                $user_hash, $remote_product_id
            ),
            ARRAY_A
        );
        
        if ( ! empty( $item_product ) ) {
            foreach ($api_response['included'] as $item ) {
                if ($item['attributes']['variant-id'] === $remote_product_id) {
                    $result = $this->wpdb->update(
                        $this->local_data_table,
                        [
                            'count_items' => $item['attributes']['quantity']
                        ],
                        [
                            'hash' => $user_hash,
                            'remote_product_id' => $remote_product_id
                        ],
                        ['%d'],
                        ['%s', '%d']
                    );
                }
            }
        } else {
            $result = $this->wpdb->insert(
                $this->local_data_table,
                [
                    'local_product_id'  => $local_product_id,
                    'remote_product_id' => $remote_product_id,
                    'count_items'       => $count,
                    'hash'              => $user_hash,
                    'order_token'       => $order_token,
                    'attributes'        => wp_json_encode( $api_response['included'] )
                ],
                [ '%d', '%d', '%d', '%s', '%s', '%s' ]
            );
        }
        
        return (bool) $result;
    }

    /**
     * @param string $order_token
     *
     * @return string
     */
    public function generate_checkout_url( $order_token ) {
        $order = $this->get_order_includes( $order_token );
        return PRODIGY_PROTOCOL_DOMAIN .
               get_option( 'pg_url_domain_hosted_system' ) . '.' .
               PRODIGY_CHECKOUT_DOMAIN  .
               '/orders/' . $order .
               '/checkout/login?token=' .
               $order_token .
               '&plugin_redirect_url=' .
               urlencode( home_url() . '/api-listener'.'?token=' . $order_token );
    }
    
    /**
     * @param string $token
     *
     * @return array
     */
    public function get_order_includes( $token ) {
        $result = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "select order_number from {$this->order_data_table} where token = %s and status <> 1",
                $token
            ),
            ARRAY_A
        );

        if (isset($result[0])) {
            return json_decode( $result[0]['order_number'], true ) ?? null;
        }

    }
    
    /**
     * @param string $user_session
     * @param string $order_token
     * @param int $remote_product_id
     *
     * @return bool
     */
    public function get_remote_product_data( $user_session, $order_token, $remote_product_id ) {
        $result = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "select * from {$this->remote_data_table} where local_user_hash = %s and token = %s and remote_product_id = %d",
                $user_session, $order_token, $remote_product_id
            ),
            ARRAY_A
        );
        
        return $result[0] ?? null;
    }
    
    /**
     * @param string $order_token
     *
     * @return array
     */
    public function get_remote_products_data( $order_token ) {
        $result = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "select * from {$this->remote_data_table} where token = %s",
                $order_token
            ),
            ARRAY_A
        );
        
        return $result ?? null;
    }

    /**
     * @param string $order_token
     * @param int    $status
     *
     * @return bool
     */
    public function update_status_orders( string $order_token, $status = self::PRODIGY_ORDER_IN_PROGRESS ) {
        
        $is_update = $this->wpdb->update(
            $this->order_data_table,
            [
                'status'     => $status,
                'updated_at' => date( "Y-m-d H:i:s", time() )
            ],
            [
                'token' => $order_token
            ],
            [ '%d', '%s' ],
            [ '%s' ]
        );
        
        return (bool) $is_update;
    }
    
    /**
     * @param string $order_token
     * @param string $user_session
     * @param array $data
     *
     * @return bool
     */
    public function update_order_data_in_storage( $order_token, $user_session, $data ) {
        $is_update = $this->wpdb->update(
            $this->order_data_table,
            [
                'includes'   => $data,
                'updated_at' => date( "Y-m-d H:i:s", time() )
            ],
            [
                'local_user_hash' => $user_session,
                'token'           => $order_token
            ],
            [ '%s', '%s' ],
            [ '%s', '%s' ]
        );
        
        return (bool) $is_update;
    }
    
    
    /**
     * @param $user_session
     * @param array $post
     * @param array $api_response
     *
     * @return bool
     */
    public function save_item_for_remote_cart( $user_session, $post, $api_response ) {
        
        $remote_product_id_request = filter_var($post['remote_product_id'], FILTER_VALIDATE_INT);
        $local_product_id_request = filter_var($post['local_product_id'], FILTER_VALIDATE_INT);
        $count_request = filter_var($post['count'], FILTER_VALIDATE_INT);
        
            $product = $this->get_remote_product_data(
            $user_session,
            $api_response['data']['attributes']['token'],
            $remote_product_id_request
        );
        
        if ( ! empty( $product ) ) {
            foreach ($api_response['included'] as $item ) {
                if ($item['attributes']['variant-id'] === $remote_product_id_request ) {
                    $result = $this->wpdb->update(
                        $this->remote_data_table,
                        [
                            'count_items' => $item['attributes']['quantity']
                        ],
                        [
                            'local_user_hash' => $user_session,
                            'token' => $api_response['data']['attributes']['token'],
                            'remote_product_id' => $remote_product_id_request
                        ],
                        ['%d'],
                        ['%s', '%s', '%d']);
                }
            }
        } else {
            $local_product_id     = $local_product_id_request;
            $remote_product_id    = $remote_product_id_request;
            $count_items          = $count_request;
            $local_user_hash      = $user_session;
            $order_number         = $api_response['data']['id'];
            $token                = $api_response['data']['attributes']['token'];
            $response             = wp_json_encode( $api_response );
            $remote_order_item_id = $api_response['data']['id'];
            $line_item_id         = $this->get_line_item( $remote_product_id, $api_response['included'] );
            $result               = $this->wpdb->insert(
                $this->remote_data_table,
                [
                    'local_product_id'     => (int) $local_product_id,
                    'remote_product_id'    => (int) $remote_product_id,
                    'local_user_hash'      => $local_user_hash,
                    'count_items'          => (int) $count_items,
                    'order_number'         => $order_number,
                    'token'                => $token,
                    'api_response'         => $response,
                    'remote_order_item_id' => $remote_order_item_id,
                    'line_item_id'         => $line_item_id
                ],
                [ '%d', '%d', '%s', '%d', '%d', '%s', '%s' ]
            );
        }
        
        return (bool) $result;
    }
    
    /**
     * @param int $remote_product_id
     * @param array $included
     *
     * @return int
     */
    private function get_line_item( $remote_product_id, $included ) {
        if ( !empty($included) ) {
            foreach ( $included as $item ) {
                if ( $item['attributes']['variant-id'] === (int) $remote_product_id ) {
                    return (int) $item['id'];
                }
            }
        }
    }
    
    /**
     * @param array $response
     * @param string $user_session
     *
     * @return bool
     */
    public function create_order( $response, $user_session ) {
        $order_number = $response['data']['id'];
        $token        = $response['data']['attributes']['token'];
        
        $result = false;
        if ( isset( $order_number ) && $token ) {
            $result = $this->wpdb->insert(
                $this->order_data_table,
                [
                    'local_user_hash' => $user_session,
                    'order_number'    => $order_number,
                    'token'           => $token,
                    'status'          => $this->get_order_status( $token ),
                    'created_at'      => date( "Y-m-d H:i:s", time() ),
                    'includes'        => wp_json_encode($response)
                ],
                [ '%s', '%d', '%s', '%d', '%s', '%s' ]
            );
        }
        
        return (bool) $result;
    }
    
    /**
     * @param string $token
     *
     * @return array|null
     */
    public function get_order_info( string $token ) {
        $completed_status = self::PRODIGY_ORDER_COMPLETED;
        
        $result = $this->wpdb->get_row(
            $this->wpdb->prepare(
                "select * from {$this->remote_data_table} where token = %s ",
                $token, $completed_status
            ),
            ARRAY_A
        );
        
        return $result;
    }
    
    /**
     * @param string $token
     *
     * @return int
     */
    public function get_order_status( $token ) {
        $completed_status = self::PRODIGY_ORDER_COMPLETED;
        
        $status = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "select * from {$this->order_data_table} where token = %s and status <> %d",
                $token, $completed_status
            ),
            ARRAY_A
        );
        
        return (int) $status;
    }
    
    /**
     * @param string $token
     *
     * @return array
     */
    public function get_in_process_order_for_token( $token ) {
        $result = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "select * from {$this->order_data_table} where token = %s and status = %d",
                $token, self::PRODIGY_ORDER_IN_PROGRESS
            ),
            ARRAY_A
        );
        
        return ! empty( $result ) ? $result[0] : false;
    }
    
    /**
     * @param string $user_hash
     *
     * @return array
     */
    public function get_cart_items_for_user( $user_hash ) {
        $cart_items = $this->wpdb->get_results(
            $this->wpdb->prepare( "
                SELECT  local_product_id, remote_product_id, count_items as count,attributes FROM {$this->local_data_table} as ldt 
                LEFT JOIN {$this->wpdb->posts} as posts on ldt.local_product_id = posts.id
                where posts.post_status = 'publish' and ldt.hash = %s
                ", $user_hash
            ),
            ARRAY_A
        );
        
        return empty( $cart_items ) ? [] : $cart_items;
    }


    /**
     * @param string $user_hash
     *
     * @return array
     */
    public function get_cart_items_for_cart_table( $user_hash ) {
        $cart_items = $this->wpdb->get_results(
            $this->wpdb->prepare( "
                SELECT  local_product_id, remote_product_id, count_items as count,attributes FROM {$this->local_data_table} where hash = %s
                ", $user_hash
            ),
            ARRAY_A
        );

        return empty( $cart_items ) ? [] : $cart_items;
    }
    
    /**
     * @param int $product_id
     * @param string $user_hash
     *
     * @return bool
     */
    public function remove_item_from_local_cart( $product_id, $user_hash ) {
        $removed_items = $this->wpdb->delete(
            $this->local_data_table, [
            'local_product_id' => $product_id,
            'hash'             => $user_hash
        ], [ '%d', '%s' ] );
        
        return (bool) $removed_items;
    }
    
    /**
     * @param int $remote_id
     * @param string $order_token
     * @param string $user_session
     *
     * @return bool
     */
    public function remove_item_from_remote_cart( $remote_id, $order_token, $user_session ) {
        $is_deleted = $this->wpdb->delete(
            $this->remote_data_table,
            [ 'remote_product_id' => $remote_id, 'token' => $order_token, 'local_user_hash' => $user_session ],
            [ '%d', '%s', '%s' ]
        );
        
        return (bool) $is_deleted;
    }
    
    
    /**
     * @param $remote_id
     * @param $count
     * @param $user_hash
     *
     * @return bool
     */
    public function update_items_from_local_cart_in_storage( $remote_id, $count, $user_hash ) {
        $result = $this->wpdb->update(
            $this->local_data_table,
            [
                'count_items' => $count
            ],
            [
                'hash'              => $user_hash,
                'remote_product_id' => $remote_id
            ],
            [ '%d' ],
            [ '%s', '%d' ]
        );
        
        return (bool) $result;
    }
    
    /**
     * @param $remote_id
     * @param $count
     * @param $user_hash
     * @param $order_token
     *
     * @return bool
     */
    public function update_items_from_remote_cart_in_storage( $remote_id, $count, $user_hash, $order_token ) {
        $result = $this->wpdb->update(
            $this->remote_data_table,
            [
                'count_items' => $count
            ],
            [
                'local_user_hash' => $user_hash,
                'token' => $order_token,
                'remote_product_id' => $remote_id
            ],
            [ '%d' ],
            [ '%s', '%s', '%d' ]
        );
        
        return (bool) $result;
    }
    
    /**
     * @param string $hash
     * @param string $date
     *
     * @return bool
     */
    public function update_expiration_date( $hash, $date ) {
        $is_update = $this->wpdb->update(
            $this->local_data_table, [ 'expiration_date' => $date ], [ 'hash' => $hash ], [
            '%s',
            '%s'
        ] );
        
        return $is_update;
    }
    
    /**
     * @param string $hash
     *
     * @return array
     */
    public function get_cart_expiration_date( $hash ) {
        $result = $this->wpdb->get_results( "select expiration_date from {$this->local_data_table} where hash = %s ", $hash );
        
        return $result;
    }

    /**
     * @param string $time
     * @param string $user_session
     *
     * @return bool
     */
    public function clean_cart_items( $user_session, $time = '0' ) {
        $result = $this->wpdb->query(
            $this->wpdb->prepare(
                "DELETE FROM {$this->local_data_table} WHERE expiration_date < %s and hash = %s",
                $time, $user_session
            )
        );
        
        return $result !== false;
    }


    /**
     * @param string $token
     *
     * @return bool
     */
    public function reset_cart_after_order_closing( $token ) {
        $result = $this->wpdb->query(
            $this->wpdb->prepare(
                "delete from {$this->local_data_table} where remote_product_id IN (
                        select remote_product_id from {$this->remote_data_table} where token = %s
                    )",
                $token
            )
        );

        return $result !== false;
    }
    
    
    /**
     * @param array $item
     * @param null $token
     *
     * @return array
     */
    public function set_cart_prepare_request( $item, $token = null ) {
        
        $count = filter_var($item['count'], FILTER_VALIDATE_INT);
        $remote_product_id = filter_var($item['remote_product_id'], FILTER_VALIDATE_INT);
        $price = filter_var($item['price'], FILTER_VALIDATE_FLOAT);
        
        $request = [
            "include"    => "line-items",
            "line_items" => [
                [
                    "quantity"   => $count,
                    "variant_id" => $remote_product_id,
                    "unit_price" => $price ?? 0
                ],
            ],
        ];

        $request["redirect_to"] = '/api-listener';

        if ( ! empty( $token ) ) {
            $request["order_token"] = $token;
        }
        
        return $request;
    }


    /**
     * @param array $remote_product
     * @param string $count
     *
     * @return array
     */
    public function prepare_line_item_request( $remote_product, $count, $price = 0 ) {
        $request = [
            "line_item"   =>
                [
                    "quantity"   => $count,
                    "variant_id" => $remote_product['remote_product_id'],
                    "unit_price" => $price
                ],
            "order_token" => $remote_product['token']
        ];
        
        return $request;
    }


    /**
     * @param string $user_session
     *
     * @return bool
     */
    public function get_token_for_current_order( $user_session ) {
        global $wpdb;
        $table_name = $this->order_data_table;
        if ( $wpdb->get_var( "SHOW TABLES LIKE '{$table_name}' " ) == $table_name ) {
            $result = $this->wpdb->get_results(
                $this->wpdb->prepare( "select token from {$this->order_data_table} where local_user_hash = %s and status <> 1 and deleted_at is null ",
                    $user_session ),
                ARRAY_A
            );
            return $result[0]['token'] ?? null;
        }

        return null;
    }


    /**
     * @param string $order_token
     *
     * @return mixed|null
     */
    public function is_empty_cart( string  $order_token ) {
        $result = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "select * from {$this->local_data_table} where order_token = %s",
                $order_token
            ),
            ARRAY_A
        );

        return empty($result);
    }

    /**
     * @param string $order_token
     *
     * @return bool
     */
    public function delete_local_order( string $order_token ) {
        $is_update = $this->wpdb->update(
            $this->order_data_table,
            [
                'deleted_at' => date( "Y-m-d H:i:s", time() )
            ],
            [
                'token' => $order_token
            ],
            [ '%s' ],
            [ '%s' ]
        );

        return (bool) $is_update;
    }


    /**
     * @param string $order_token
     *
     * @return bool
     */
    public function is_deleted_order( string $order_token ) {
        $result = $this->wpdb->get_row(
            $this->wpdb->prepare(
                "select deleted_at from {$this->order_data_table} where token = %s",
                $order_token
            ),
            ARRAY_A
        );

        return !empty($result['deleted_at']);
    }


}
