<?php

/**
 * Prodigy product variants class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Product_Variants
 */
class Prodigy_Product_Variants extends Prodigy_Product {


    public static $variant_post_type = Prodigy::PRODIGY_VARIATION_POST_TYPE;

    public static $check_attribute_list = [
        'check_stock_status',
        'check_requires_shipping',
        'check_sale_price'
    ];

    /**
     * Generate variants from attributes
     *
     * @param array $enablers
     * @param array $attributes
     *
     * @return array
     */
    public function prepare_variants( $enablers, $attributes ) {
        $variants = [];

        if ( !empty( $attributes ) ) {
            foreach ( $attributes as $key => $item ) {
                if ( sanitize_text_field($enablers[$key]) == 'on' ) {
                    $variants[reset($item['attribute'])] = $item['values'];
                }
            }
        }

        return $variants;
    }

    /**
     * Generate variants combination
     *
     * @param array $variants
     *
     * @return array
     */
    public function generate_variants( $variants ) {

        $variant_keys = array_keys( $variants  );
        $first = array_shift( $variants );
        $attrName = array_shift($variant_keys);
        $variant_list = $this->get_all_array_combinations( $attrName, $first, $variants );
        $names = $this->convert_to_string( $variant_list );

        $response = [];
        foreach ( $names as $key => $name ) {
            if (isset($variant_list[0])) {
                $response[] = [
                    'name' => $name,
                    'attrs' => isset($variant_list[0]) ? $variant_list[$key] : $variant_list[$name],
                ];
            } else {
                $response[$key]['name'] = $name;
                $response[$key]['attrs'][$variant_list[$name]] = $name;
            }
        }

        return $response;
    }

    /**
     * @param array $list_variants
     *
     * @return array
     */
    private function convert_to_string( $list_variants ) {
        $response = [];
        if ( count( $list_variants ) != count( $list_variants, COUNT_RECURSIVE ) ) {
            $array = [];
            $i = 0;
            foreach ( $list_variants as $variant ) {
                $array[$i++] = implode( "|", $variant );
            }
             $response = $array;
        } else {
            $i = 0;
            foreach ($list_variants as $key=>$variants) {
                $response[$i] = $key;
                $i++;
            }
        }

        return $response;
    }

    /**
     * @param string $attr_name1
     * @param array $list1
     * @param array $list2
     *
     * @return array
     */
    private function get_all_array_combinations( $attr_name1, $list1, $list2 )
    {
        if ( empty( $list1 ) ) {
            return $list2;
        }

        if ( empty( $list2 ) ) {
            $new_list1 = [];
            foreach ($list1 as $val) {
                $new_list1[$val] = $attr_name1;
            }
            return $new_list1;
        }

        $list2_keys = array_keys($list2);
        if ( isset($list2_keys[0] ) && is_array( $list2[$list2_keys[0]] ) ) {
            $attr_name2 = $list2_keys[0];
            $first = array_shift( $list2 );
            $list2 = $this->get_all_array_combinations( $attr_name2, $first, $list2 );
        }
        $combined = [];

        foreach ( $list1 as $value1 ) {
            foreach ( $list2 as $value2=>$attr2 )  {
                $combined[] = is_array( $attr2 ) ? array_merge( [$attr_name1 => $value1], $attr2 ) : [$attr_name1 => $value1, $attr2 => $value2];
            }
        }

        return $combined;
    }

    /**
     * @param array $product
     * @param array $variants
     *
     * @return array
     */
    public function save( $product, $variants ) {
        $result = [];
        $count_variants = count( $variants['post_title'] );

        for ( $i=0; $i < $count_variants; $i++ ) {
            $this->save_variant( $product, $i, $variants );
        }

        return $result;
    }

    /**
     * @param object $product
     * @param string $index
     * @param array $variants
     *
     * @return int
     */
    private function save_variant( $product, $index, $variants )
    {
        global $wpdb;

        $result = $wpdb->insert(
            $wpdb->posts,
            [
                'post_type' => self::$variant_post_type,
                'post_status' => $product->post_status,
                'post_author' => (int) $product->post_author,
                'post_title' => $variants['post_title'][$index],
                'post_content' => '',
                'post_parent' => $product->ID,
                'comment_status' => 'closed',
                'post_date' => gmdate( 'Y-m-d H:i:s' ),
                'post_date_gmt' => gmdate( 'Y-m-d H:i:s' ),
                'post_name' => $this->generate_slug( $variants['post_title'][$index] )
            ],
            [ '%s', '%s', '%d', '%s', '%s', '%d', '%s', '%s', '%s', '%s' ]
        );

        $post_id = $wpdb->insert_id;

        if ( $result ) {
            foreach ( $variants as $key => $variant ) {
                update_post_meta( $post_id, $key, $variant[$index] );
            }
        }

        return $result;
    }

    /**
     * @param object $parent_product
     * @param array $current_variants
     * @param array $new_variants
     */
    public function update_variants( $parent_product, $current_variants, $new_variants )
    {
        $current_variants_titles = [];
        foreach ( $current_variants as $variant ) {
            $current_variants_titles[$variant['post_title']] = [
                'delete_item' => true,
                'id' => $variant['ID']
            ];
        }

        $index = 0;
        foreach ( $new_variants['post_title'] as $key => $new_variant_title ) {
            if ( isset( $current_variants_titles[$new_variant_title] ) ) {
                $current_variants_titles[$new_variant_title]['delete_item'] = false;
                $this->update_variant( $current_variants_titles[$new_variant_title]['id'], $key, $new_variants );
            } else {
                $this->save_variant( $parent_product, $index, $new_variants );
            }
            $index++;
        }
        $current_variants_titles = array_keys( array_filter( $current_variants_titles, function( $item ) {
            return $item['delete_item'];
            }
        ));

        $this->delete_variants( $parent_product, $current_variants_titles );
    }

    /**
     * @param int $old_variant_id
     * @param string $new_key_variant
     * @param array $new_variants
     */
    private function update_variant( $old_variant_id, $new_key_variant, $new_variants ) {
        $new_variant = [];

        foreach ( $new_variants as $key => $variant ) {
            if ( isset($variant[$new_key_variant]) ) {
                $new_variant[$key] = $variant[$new_key_variant];
            }
        }

        foreach ( self::$check_attribute_list as $key => $item ) {
            $check_options = array_search( $item, array_keys( $new_variant ) );
            if ( !$check_options ) {
                delete_post_meta( $old_variant_id, $item );
            }
        }

        foreach ( $new_variant as $key => $variant ) {
            update_post_meta( $old_variant_id, $key, $variant );
        }
    }

    /**
     * @param object $parent_product
     * @param array $variants_titles
     */
    private function delete_variants( $parent_product, $variants_titles ) {
        global  $wpdb;

        foreach ( $variants_titles as $title ) {
            $variant_id = $wpdb->get_var( $wpdb->prepare(
                "SELECT * FROM {$wpdb->posts} WHERE post_parent= %d and post_title='%s' and post_type='%s' ",
                [ $parent_product->ID, $title, self::$variant_post_type ]
            ));
            $result = wp_delete_post( $variant_id );

            if ($result) {
                $wpdb->delete( $wpdb->postmeta, ['post_id' => (int) $variant_id], ['%d'] );

                update_option('prodigy_cart_removed_products', $variant_id);
                update_option('prodigy_widget_cart_removed_products', $variant_id);
                wp_update_post(['ID' => $variant_id, 'post_status' => 'trash']);
            }
        }
    }

    /**
     * @param int $parent_post_id
     *
     * @return array
     */
    public static function get_variants_from_product( $parent_post_id ) {
        global $wpdb;
        $post_type = Prodigy::PRODIGY_VARIATION_POST_TYPE;
        $variants = $wpdb->get_results( $wpdb->prepare(
            "SELECT * FROM {$wpdb->posts} WHERE post_parent=%d and post_type='%s' ",
            [ $parent_post_id, $post_type ]
        ), ARRAY_A );

        $meta_variants = [];
        foreach ( $variants as $key=>$variant ) {
            $meta_variants[$key] = get_post_meta( $variant['ID'] );
        }

        $count = count( $variants );
        $variants_array = [];
        for ( $i=0; $i < $count; $i++ ) {
            $variants_array[$i] = array_merge( $variants[$i], self::format_meta_array( $meta_variants[$i] ) );
        }

        return $variants_array;
    }

    /**
     * @param int $parent_id
     *
     * @return array|object|null
     */
    public static function get_product_variants( $parent_id ) {
        global $wpdb;

        $post_type = Prodigy::PRODIGY_VARIATION_POST_TYPE;

        $variants = $wpdb->get_results( $wpdb->prepare(
            "SELECT * FROM {$wpdb->posts} WHERE post_parent=%d and post_type='%s' ",
            [ $parent_id, $post_type ]
        ), ARRAY_A );

        if ( empty( $variants ) ) {
            $variants = [];
        }

        return $variants;
    }

    /**
     * @param int $parent_post_id
     *
     * @return array
     */
    public function delete_all_variants( $parent_post_id ) {
        global $wpdb;

        $result = $wpdb->get_results( $wpdb->prepare(
            "DELETE FROM {$wpdb->posts} WHERE post_parent=%d and post_type='%s'",
            [ $parent_post_id, self::$variant_post_type ]
        ));

        return $result;
    }

    /**
     * @param string $parent_sku
     * @param string $variant_name
     *
     * @return string
     */
    public function generate_sku( $parent_sku, $variant_name ) {
        $parse_name = explode("|", $variant_name );
        $count = count($parse_name);

        $variant_sku = $parent_sku;
        for ( $i = 0; $i < $count; $i++ ) {
            $variant_sku .= '-'. $parse_name[$i];
        }

        return $variant_sku;
    }

    /**
     * @param int $parent_id
     * @param string $name
     *
     * @return object|null
     */
    public function get_product_variant_by_name( $parent_id, $name ) {
        global $wpdb;

        $variant = $wpdb->get_row( $wpdb->prepare(
            "SELECT * FROM {$wpdb->posts} WHERE post_parent= %d and post_title='%s' and post_type='%s' ",
            [ $parent_id, $name, self::$variant_post_type ]
        ));

        return $variant;
    }

}
