<?php

include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-api-client.php';

/**
 * The core plugin class.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */
class Prodigy {

    const PLUGIN_NAME = 'prodigy';
    const PRODIGY_POST_TYPE_DEFAULT = 'product';
    const PRODIGY_VARIATION_POST_TYPE = 'prodigy_variation';
    const PRODIGY_TAXONOMY_CATEGORY_DEFAULT = 'product-category';
    const PRODIGY_DEFAULT_CATEGORY_OPTION = 'default_prodigy_product_cat';
    const PRODIGY_DEFAULT_NAME_CATEGORY = 'DefaultCategory';
    const PRODIGY_TAXONOMY_TAG_DEFAULT = 'product-tag';
    const PRODIGY_UNSYNCHRONIZED_POST_STATUS = 'unsynchronized';
    const PRODIGY_PUBLISH_POST_STATUS = 'publish';
    const PRODIGY_HIDE_STATUS = 'out_of_stock';
    const PRODIGY_PROTOCOL_HOSTED_SYSTEM = 'https://';
    const PRODIGY_FIRST_CONNECTED_PRODUCT = 'pg_first_connected_product';
    
    const PRODIGY_OPTION_NAME_OLD_TAG_TYPES = 'pg_all_slugs_tag_type';
    const PRODIGY_OPTION_NAME_OLD_PRODUCT_TYPES = 'pg_all_slugs_product_type';
    const PRODIGY_OPTION_NAME_OLD_CATEGORY_TYPES = 'pg_all_slugs_category_type';
    
    const PRODIGY_OPTION_NAME_CURRENT_PRODUCT_TYPE = 'pg_slug_product_type';
    const PRODIGY_OPTION_NAME_CURRENT_CATEGORY_TYPE = 'pg_slug_category_type';
    const PRODIGY_OPTION_NAME_CURRENT_TAG_TYPE = 'pg_slug_tag_type';
    
    const PRODIGY_ACCESS_UPDATE_TYPE_PRODUCTS_QUEUE = 'pg_access_update_type_products_queue';
    const PRODIGY_ACCESS_UPDATE_TYPE_CATEGORIES_QUEUE = 'pg_access_update_type_categories_queue';
    const PRODIGY_ACCESS_UPDATE_TYPE_TAGS_QUEUE = 'pg_access_update_type_tags_queue';
    
    const PRODIGY_REDIRECT_AFTER_UPDATE_TYPES = 'pg_redirect_after_update_types';
    
    const APPEARANCE_MENUS_CHECKED_FIRST = 3;
    
    const PRODIGY_INDICATOR_SYNC_CONTENT = 'pg_indicator_sync_content';


    const PAGE_PRODIGY_ATTRIBUTES = '_page_prodigy_attributes';
    const PAGE_PRODIGY_SETTINGS = '_page_prodigy_settings';
    
    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $plugin_name The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $version The current version of the plugin.
     */
    protected $version;

    protected $api_client;

    /**
     * Define the core functionality of the plugin.
     *
     * @since    1.0.0
     */
    public function __construct() {
        global $wpdb;

		if ( defined( 'PRODIGY_VERSION' ) ) {
			$this->version = PRODIGY_VERSION;
		}
		$this->plugin_name = self::PLUGIN_NAME;

        $this->load_dependencies();

        $table_name = $wpdb->prefix . 'prodigy_attribute_taxonomy';
        if ( $wpdb->get_var( "SHOW TABLES LIKE '{$table_name}' " ) != $table_name ) {
            Prodigy_Install_Wizard::set_tables();
        }

        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();

        $this->set_ajax();
        $this->admin_update_category();
        $this->admin_update_tag();
        $this->set_analytics_key();

        add_action( 'init', [ $this, 'register_post_types' ] );
        add_action( 'init', [ $this, 'register_taxonomies' ] );
        add_action( 'init', [ $this, 'register_tags' ] );
        add_action( 'init', [ $this, 'register_new_post_types' ] );
        add_action( 'init', [ $this, 'includes' ] );
        add_action( 'init', [ $this, 'update_connection_url' ] );
        add_action( 'init', [ $this, 'update_content_by_change_types' ] );
        add_action( 'init', [ $this, 'add_default_taxonomy' ] );
        add_action( 'init', [ $this, 'create_pages' ] );
        add_action( 'init', [ $this, 'loadGlobalScripts' ] );
        
        $shop_page_uri = parse_url( $_SERVER["REQUEST_URI"], PHP_URL_PATH );
        if ( ! is_admin() && !substr_count( $shop_page_uri, "api-listener" ) ) {
            add_action( 'init', [ $this, 'prodigy_set_user_session' ] );
        }

        add_post_type_support( self::get_prodigy_product_type(), 'thumbnail' );

        // Enable shortcodes in text widgets
        add_filter( 'widget_text', 'do_shortcode' );
        add_action( 'activated_plugin', [$this, 'prodigy_activation_redirect'] );
        add_filter( 'get_user_option_metaboxhidden_nav-menus', [ $this, 'custom_display_appearance_menus'], 10, 3 );
        
        $this->set_old_type_prodigy_product();
        $this->set_old_type_prodigy_category();
        $this->set_old_type_prodigy_tag();
        $this->set_option_sync_content_wizard();
    
        /**
         * Migration BD
         */
        add_action( 'init', [ $this, 'check_update_bd' ], 9 );
        
        add_action( 'init', [ $this, 'registerOldTypes' ] );
    }
    
    /**
     * Register all old types
     */
    public function registerOldTypes() {

        $list_types_post = get_option('pg_all_slugs_product_type');

        if (is_array($list_types_post)) {
            Prodigy_Type::registerAllTypes($list_types_post);
        }
    }
    
    /**
     * Load global JS
     */
    public function loadGlobalScripts()
    {
        wp_register_script(
            'global',
            dirname( plugin_dir_url(__FILE__) ) . '/web/global/global.js',
            array( 'jquery' )
        );
        wp_enqueue_script( 'global' );
    
        wp_localize_script('global', 'prodigyajax',
            [
                'url' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('prodigyajax-nonce')
            ]
        );
    }

    /**
     * Set google analytics key
     */
    public function set_analytics_key() {
        $this->api_client = new Prodigy_Api_Client();
        $result           = $this->api_client->get_social_account_key();
        $response_body    = json_decode( wp_remote_retrieve_body( $result ), true );

        if ( ! isset( $response_body['errors'] ) ) {
            $option = get_option('pg_google_track_id');

            if (isset($response_body['data']['attributes']['external-key'])) {
                $api_option = $response_body['data']['attributes']['external-key'];
                if ( $option !== $api_option ) {
                    update_option( 'pg_google_track_id', $api_option );
                }
            }
        } else {
            if ( ! empty( $response_body['errors'][0] ) ) {
                delete_option( 'pg_google_track_id');
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger', $response_body['errors'][0]['detail'], 'error' );
                }
            }
        }
    }
    
    /**
     * Update BD by update plugin
     */
    public function check_update_bd() {
        if ( ProdigyUpdaterMigration::checkCurrentVersionMigration() ) {
            $updater = new ProdigyUpdaterMigration();
            return $updater->executeMigration();
        }
    }
    
    /**
     * Get current prodigy product type (slug)
     * @return string
     */
    public static function get_prodigy_product_type() {
        return ( new Prodigy_Type( self::PRODIGY_POST_TYPE_DEFAULT ) )->get_type();
    }
    
    /**
     * Get current prodigy category type (slug)
     * @return string
     */
    public static function get_prodigy_category_type() {
        return ( new Prodigy_Type( self::PRODIGY_TAXONOMY_CATEGORY_DEFAULT ) )->get_type();
    }
    
    /**
     * Get current prodigy tag type (slug)
     * @return string
     */
    public static function get_prodigy_tag_type() {
        return ( new Prodigy_Type( self::PRODIGY_TAXONOMY_TAG_DEFAULT ) )->get_type();
    }
    
    /**
     * check old type product
     */
    public function set_old_type_prodigy_product() {
        $types = get_option( Prodigy::PRODIGY_OPTION_NAME_OLD_PRODUCT_TYPES );
        if ( empty( $types ) ) {
            update_option( Prodigy::PRODIGY_OPTION_NAME_OLD_PRODUCT_TYPES, [ 'prodigy_product_type', 'product' ] );
        }
    
        $type = get_option( Prodigy::PRODIGY_OPTION_NAME_CURRENT_PRODUCT_TYPE );
        if ( empty( $type ) ) {
            update_option( Prodigy::PRODIGY_OPTION_NAME_CURRENT_PRODUCT_TYPE, self::PRODIGY_POST_TYPE_DEFAULT );
        }
    }
    
    /**
     * check old type product
     */
    public function set_option_sync_content_wizard() {
        $types = get_option( Prodigy::PRODIGY_INDICATOR_SYNC_CONTENT );
        if ( empty( $types ) ) {
            update_option( Prodigy::PRODIGY_INDICATOR_SYNC_CONTENT, 'yes' );
        }
    }
    
    /**
     * check old type product
     */
    public function set_old_type_prodigy_category() {
        $types = get_option( Prodigy::PRODIGY_OPTION_NAME_OLD_CATEGORY_TYPES );
        if ( empty( $types ) ) {
            update_option( Prodigy::PRODIGY_OPTION_NAME_OLD_CATEGORY_TYPES, [ 'prodigy_product_cat', 'product-category' ] );
        }
    
        $type = get_option( Prodigy::PRODIGY_OPTION_NAME_CURRENT_CATEGORY_TYPE );
        if ( empty( $type ) ) {
            update_option( Prodigy::PRODIGY_OPTION_NAME_CURRENT_CATEGORY_TYPE, self::PRODIGY_TAXONOMY_CATEGORY_DEFAULT );
        }
    }
    
    /**
     * check old type product
     */
    public function set_old_type_prodigy_tag() {
        $types = get_option( Prodigy::PRODIGY_OPTION_NAME_OLD_TAG_TYPES );
        if ( empty( $types ) ) {
            update_option( Prodigy::PRODIGY_OPTION_NAME_OLD_TAG_TYPES, [ 'prodigy_product_tag', 'product-tag' ] );
        }
    
        $type = get_option( Prodigy::PRODIGY_OPTION_NAME_CURRENT_TAG_TYPE );
        if ( empty( $type ) ) {
            update_option( Prodigy::PRODIGY_OPTION_NAME_CURRENT_TAG_TYPE, self::PRODIGY_TAXONOMY_TAG_DEFAULT );
        }
    }
    
    /**
     * @return string
     */
    public static function get_default_prodigy_product_type() {
        return self::PRODIGY_POST_TYPE_DEFAULT;
    }
    
    /**
     * Product type prodigy_product_type -> product
     * Category type prodigy_product_cat -> product-category
     * Tag type prodigy_product_tag -> product-tag
     */
    public function update_content_by_change_types() {
        $update_content = get_option( 'pg_update_content_replace_types' );
        if ( $update_content < 2 ) {
            $this->update_all_products();
            $this->update_all_categories();
            $this->update_all_tags();
            
            update_option( 'pg_update_content_replace_types', $update_content++ );
        }
    }
    
    /**
     * @return bool
     */
    private function update_all_products() {
        $args = array(
            'post_type'   => 'prodigy_product_type',
            'numberposts' => - 1
        );
        
        $pages = get_posts( $args );
        
        if ( empty( $pages ) ) {
            return true;
        }
        foreach ( $pages as $page ) {
            $post = array(
                'ID'        => $page->ID,
                'post_type' => 'product',
            );
            wp_update_post( $post );
        }
    }
    
    /**
     * @return bool
     */
    private function update_all_categories() {
        
        global $wpdb;
        
        $old_terms = $wpdb->get_results( $wpdb->prepare(
            "select * from {$wpdb->term_taxonomy} tt left join {$wpdb->terms} t on t.term_id = tt.term_id where tt.taxonomy='%s'",
            [ 'prodigy_product_cat' ]
        ) );
        
        if ( empty( $old_terms ) ) {
            return true;
        }
        
        foreach ( $old_terms as $old_term ) {
            $wpdb->update( $wpdb->term_taxonomy, [ 'taxonomy' => 'product-category' ], [ 'term_taxonomy_id' => $old_term->term_taxonomy_id ] );
        }
    }
    
    /**
     * @return bool
     */
    private function update_all_tags() {
        global $wpdb;
        
        $old_terms = $wpdb->get_results( $wpdb->prepare(
            "select * from {$wpdb->term_taxonomy} where taxonomy='%s'",
            [ 'prodigy_product_tag' ]
        ) );
        
        if ( empty( $old_terms ) ) {
            return true;
        }
        
        foreach ( $old_terms as $old_term ) {
            $wpdb->update( $wpdb->term_taxonomy, [ 'taxonomy' => 'product-tag' ], [ 'term_taxonomy_id' => $old_term->term_taxonomy_id ] );
        }
    }
    
    /**
     * After installing the plugin, set the default field "Product categories" to active appearance->menus->screen-options
     *
     * @param $result
     * @param $option
     * @param $user
     *
     * @return array
     */
    public function custom_display_appearance_menus( $result, $option, $user ) {
        $current_type_category = Prodigy::get_prodigy_category_type();
        if ( get_option( 'pg_appearance_menus_checked_first' ) < self::APPEARANCE_MENUS_CHECKED_FIRST ) {
            $prodigy_appearance_menus = (int) get_option( 'pg_appearance_menus_checked_first' );
            
            ?>
            <script type="text/javascript">
                let type_category = <?php echo $current_type_category; ?>;
                jQuery(document).ready(function ($) {
                    $('#add-'+type_category+'-hide').attr('checked', true);
                });
            </script>
            <?php
            
            if ( in_array( 'add-'.$current_type_category, $result ) ) {
                $result = array_diff( $result, array( 'add-'.$current_type_category ) );
                update_user_meta( $user->ID, 'metaboxhidden_nav-menus', $result );
            }
            
            update_option( 'pg_appearance_menus_checked_first', ++$prodigy_appearance_menus );
        }
        
        return $result;
    }

    public function create_pages() {
        global $wpdb;
    
        $prodigy_pages = array(
            'shop'      => array(
                'name'    => _x( 'shop', 'Page slug', 'prodigy' ),
                'title'   => _x( 'Shop', 'Page title', 'prodigy' ),
                'content' => '',
            ),
            'cart'      => array(
                'name'    => _x( 'cart', 'Page slug', 'prodigy' ),
                'title'   => _x( 'Cart', 'Page title', 'prodigy' ),
                'content' => '[prodigy_cart]',
            ),
            'thank-you'      => array(
                'name'    => _x( 'thank-you', 'Page slug', 'prodigy' ),
                'title'   => _x( 'Thank you', 'Page title', 'prodigy' ),
                'content' => '',
            )
        );
        
        $pg_set_pages = get_option( 'pg_set_pages_shortcode' );
        if ( $pg_set_pages ) {
            return true;
        }
        
        $content_short_cart = false;
        
        foreach ($prodigy_pages as $page) {
    
            $prodigy_page = $this->get_page_by_name(  $page['name'] );
            
            if (!empty($prodigy_page) && $page['name'] == 'cart') {
                $pos = strpos($prodigy_page->post_content, $page['content']);
                if ($pos === false) {
                    $content_short_cart = true;
                }
            }
            
            if ( empty($pg_set_pages) && empty( $prodigy_page ) ) {
    
                $page_data = array(
                    'post_status'    => 'publish',
                    'post_type'      => 'page',
                    'post_author'    => 1,
                    'post_name'      => $page['name'],
                    'post_title'     => $page['title'],
                    'post_content'   => $page['content'],
                    'comment_status' => 'closed',
                );
                
                $wpdb->insert( $wpdb->posts, $page_data, [
                    '%s',
                    '%s',
                    '%d',
                    '%s',
                    '%s',
                    '%s',
                    '%s'
                ] );
            } else {
                if ( empty($pg_set_pages) && $prodigy_page->post_status != 'publish') {
                        $wpdb->query($wpdb->prepare(
                            "UPDATE $wpdb->posts SET post_status = %s WHERE ID = %d",
                            [ $prodigy_page->post_status, $prodigy_page->ID ]
                        )
                    );
                }
                
                if ($content_short_cart && $page['name'] == 'cart') {
                    $wpdb->query($wpdb->prepare(
                        "UPDATE $wpdb->posts SET post_content = %s WHERE ID = %d",
                        [ $prodigy_page->post_content . ' ' . $page['content'], $prodigy_page->ID ]
                        )
                    );
                }
            }
        }

        update_option( 'pg_set_pages_shortcode', true );

        // TODO set blog static page. Need to enable after install blog
        //update_option( 'show_on_front', 'page' );
        //$blog = get_page_by_title( 'Blog' );
//        update_option( 'page_for_posts', $blog->ID );

        Prodigy::refresh_permalink();
    }

    /**
     * @param string $name
     * @return array
     */
    public function get_page_by_name( $name ) {
        global $wpdb;
        $result = false;
        if ( isset($name) ) {
            $result = $wpdb->get_row( $wpdb->prepare(
                "SELECT * FROM {$wpdb->posts} as product WHERE post_type='page' and post_name = '%s'",
                [ $name ]
            ));
        }
        
        return $result;
    }
    
    /**
     * Redirect on wizard after activated plugin
     * @param $plugin
     */
    public function prodigy_activation_redirect( $plugin ) {
        if( $plugin === PRODIGY_PLUGIN_DIRECTORY_NAME . '/prodigy-commerce.php' ) {
            exit( wp_redirect( admin_url( 'index.php?page=prodigy-setup' ) ) );
        }
    }

    /**
     * Update plugin url
     */
    public function update_connection_url() {

        $site_url = UrlHelper::getSiteUrl(site_url());

        if ( get_option( 'pg_connection_url') !== $site_url ) {
            update_option( 'pg_connection_url' , $site_url );
            $api_client = new Prodigy_Api_Client();
            $response = $api_client->update_connection_url( $site_url );
            if ( $response['response']['code'] != 204 ) {
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger', $response['response'], 'error' );
                }
            }
        }
    }

    /**
     * Set user cookie
     */
    public function prodigy_set_user_session() {
        // set cart session
        $cart = new Prodigy_Cart();
        $cart->set_user_session( sanitize_text_field($_COOKIE['prodigy_user']));
        $cart->clean_cart_items( sanitize_text_field($_COOKIE['prodigy_user']), time() );
    }

    /**
     * Get the plugin url.
     *
     * @return string
     */
    public static function plugin_url() {
        return untrailingslashit( plugins_url( '/', PRODIGY_PLUGIN_FILE ) );
    }

    public function includes() {

        // Wizard.
        if ( isset( $_GET['page'] ) ) {
            switch ( $_GET['page'] ) {
                case 'prodigy-setup':
                    require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-admin-setup-wizard.php';
                    break;
            }
        }
        
        if ( isset($_GET['prodigy-clear-wizard'] ) && $_GET['prodigy-clear-wizard'] == 1) {
            update_option('pg_step_wizard', 1);
        }
    }

    /**
     * Register custom post type
     */
    public function register_new_post_types() {
        register_post_status( self::PRODIGY_UNSYNCHRONIZED_POST_STATUS, [
            'label'                     => _x( 'Unsynchronized', 'post' ),
            'public'                    => false,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'label_count'               => _n_noop(
                'Unsynchronized <span class="count">(%s)</span>',
                'Unsynchronized <span class="count">(%s)</span>'
            ),
        ] );

        register_post_status( self::PRODIGY_HIDE_STATUS, [
            'label'                     => _x( 'Hidden', 'post' ),
            'public'                    => false,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'label_count'               => _n_noop(
                'Hidden <span class="count">(%s)</span>',
                'Hidden <span class="count">(%s)</span>'
            ),
        ] );
    }

    /**
     * Register prodigy post type
     */
    public function register_post_types() {
        register_post_type( self::get_prodigy_product_type(), [
            'label'        => __( 'Prodigy', 'prodigy' ),
            'labels'       => [
                'name'           => __( 'Prodigy', 'prodigy' ),
                'all_items'      => __( 'All Products', 'prodigy' ),
                'singular_name'  => __( 'Prodigy Item', 'prodigy' ),
                'add_new'        => __( 'Add Product', 'prodigy' ),
                'add_new_item'   => __( 'Add Product', 'prodigy' ),
                'edit_item'      => __( 'Edit Product', 'prodigy' ),
                'menu_name'      => __( 'Prodigy', 'prodigy' ),
                'featured_image' => __( 'Product Image', 'prodigy' ),
                'search_items'   => __( 'Search Products', 'prodigy' ),
            ],
            'description'  => '',
            'public'       => true,
            'hierarchical' => false,
            'supports'     => [ 'title', 'excerpt', 'editor', 'thumbnail', 'comments' ],
            'taxonomies'   => [],
            'has_archive'  => false,
            'rewrite'      => true,
            'query_var'    => true,
            'menu_icon'    => 'dashicons-prodigy-logo'
        ] );

        register_post_type(
            self::PRODIGY_VARIATION_POST_TYPE,
            [
                'label'        => __( 'Variations', 'prodigy' ),
                'public'       => false,
                'hierarchical' => false,
                'supports'     => false,
                'rewrite'      => false,
            ]
        );
    }

    /**
     * Register prodigy taxonomies.
     */
    public function register_taxonomies() {

        if ( ! is_blog_installed() ) {
            return;
        }

        if ( taxonomy_exists( self::get_prodigy_category_type() ) ) {
            return;
        }

        $labels = [
            'name'              => __( 'Product Categories', 'prodigy' ),
            'singular_name'     => __( 'Category', 'prodigy' ),
            'menu_name'         => __( 'Categories', 'prodigy' ),
            'search_items'      => __( 'Search Categories', 'prodigy' ),
            'all_items'         => __( 'All Categories', 'prodigy' ),
            'parent_item'       => __( 'Parent Category', 'prodigy' ),
            'parent_item_colon' => __( 'Parent Category', 'prodigy' ),
            'edit_item'         => __( 'Edit Category', 'prodigy' ),
            'update_item'       => __( 'Update Category', 'prodigy' ),
            'add_new_item'      => __( 'Add New Category', 'prodigy' ),
            'new_item_name'     => __( 'New Category Name', 'prodigy' ),
            'not_found'         => __( 'No Categories Found', 'prodigy' ),
        ];

        $args = [
            'hierarchical'      => true,
            'show_ui'           => true,
            'show_admin_column' => true,
            'query_var'         => true,
            'label'             => __( 'Categories', 'prodigy' ),
            'labels'            => $labels,
            'capabilities'      => array(
                'pg_manage_terms' => 'pg_manage_product_terms',
            ),
        ];

        register_taxonomy( self::get_prodigy_category_type(), [ self::get_prodigy_product_type() ], $args );
        
        $attributes = Prodigy_Product_Attributes::get_attribute_taxonomies();

        if ( ! empty( $attributes ) ) {
            foreach ( $attributes as $attribute ) {

                $args = [
                    'hierarchical'      => false,
                    'show_ui'           => false,
                    'show_admin_column' => false,
                    'query_var'         => true,
                ];

                register_taxonomy( $attribute->slug, [ Prodigy::get_prodigy_product_type() ], $args );
            }
        }
    }

    /**
     * Add default category taxonomy
     */
    public function add_default_taxonomy() {
        $prodigy_default_category = (int) get_option( self::PRODIGY_DEFAULT_CATEGORY_OPTION, 0 );

        if ( empty(term_exists($prodigy_default_category)) ) {
            $default_product_cat_id   = 0;
            $default_product_cat_slug = sanitize_title( _x( Prodigy::PRODIGY_DEFAULT_NAME_CATEGORY,
                'Default category slug', 'prodigy' ) );
            $default_product_cat      = get_term_by( 'slug', $default_product_cat_slug,
                Prodigy::get_prodigy_category_type() );

            if ( $default_product_cat ) {
                $default_product_cat_id = absint( $default_product_cat->term_taxonomy_id );
            } else {
                $result = wp_insert_term( _x( Prodigy::PRODIGY_DEFAULT_NAME_CATEGORY, 'Default category slug',
                    'prodigy' ), Prodigy::get_prodigy_category_type(), array( 'slug' => $default_product_cat_slug ) );

                if ( ! is_wp_error( $result ) && ! empty( $result['term_taxonomy_id'] ) ) {
                    $default_product_cat_id = absint( $result['term_taxonomy_id'] );
                }
            }

            if ( $default_product_cat_id ) {
                update_option( self::PRODIGY_DEFAULT_CATEGORY_OPTION, $default_product_cat_id );
            }
        }
    }

    /**
     *  Register prodigy tags
     */
    public function register_tags() {

        $labels = [
            'name'                       => __( 'Product Tags', 'prodigy' ),
            'singular_name'              => __( 'Tag', 'prodigy' ),
            'menu_name'                  => __( 'Tags', 'prodigy' ),
            'search_items'               => __( 'Search Tags', 'prodigy' ),
            'all_items'                  => __( 'All Tags', 'prodigy' ),
            'edit_item'                  => __( 'Edit Tag', 'prodigy' ),
            'update_item'                => __( 'Update Tag', 'prodigy' ),
            'add_new_item'               => __( 'Add New Tag', 'prodigy' ),
            'new_item_name'              => __( 'New Tag Name', 'prodigy' ),
            'popular_items'              => __( 'Popular Tags', 'prodigy' ),
            'separate_items_with_commas' => __( 'Separate tags with commas', 'prodigy' ),
            'add_or_remove_items'        => __( 'Add or remove tags', 'prodigy' ),
            'choose_from_most_used'      => __( 'Choose from the most used tags', 'prodigy' ),
            'not_found'                  => __( 'No tags found', 'prodigy' ),
        ];

        $args = [
            'hierarchical'      => false,
            'show_ui'           => true,
            'show_admin_column' => true,
            'query_var'         => true,
            'label'             => __( 'Product tags', 'prodigy' ),
            'labels'            => $labels,
            'capabilities'      => array(
                'pg_manage_terms' => 'pg_manage_product_terms',
            ),
        ];

        register_taxonomy( self::get_prodigy_tag_type(), [ self::get_prodigy_product_type() ], $args );
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Prodigy_Loader. Orchestrates the hooks of the plugin.
     * - Prodigy_Admin. Defines all hooks for the admin area.
     * - Prodigy_Public. Defines all hooks for the public side of the site.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         *  Add support template functions
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/prodigy-global.php';
    
        /**
         * Migration
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/migrations/ProdigyUpdaterMigration.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/migrations/InterfaceProdigyMigration.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/migrations/ProdigyMigration107.php';
        
        /**
         * Helpers
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/helpers/class-prodigy-helper-hosted-system.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/helpers/UrlHelper.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/helpers/helpers-shortcode.php';

        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-install-wizard.php';

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-i18n.php';

        /**
         * Set order category ajax class
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-ajax.php';

        /**
         * Product entity class
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-product.php';

        /**
         * Product comments class
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-product-comments.php';

        /**
         * Add API client
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-api-client.php';

        /**
         * Class template loader
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/class-prodigy-template-loader.php';


        /**
         *  Add support template functions
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/prodigy-template-functions.php';

        /**
         * Set template hooks
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/prodigy-template-hooks.php';


        ///// Shortcodes /////
    
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-common.php';

        /**
         * Products attributes
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-attributes-filter.php';

        /**
         * Products price filter
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-price-filter.php';

        /**
         * Active filters
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-active-filters.php';

        /**
         * Categories filters
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-left-categories-filters.php';

        /**
         * Breadcrumbs list
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-breadcrumbs-list.php';

        /**
         * Search
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-search.php';

        /**
         * My Account
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-my-account.php';
    
        /**
         * cart
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-cart.php';

        /**
         * Render cart page
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/pages/class-prodigy-shortcode-cart-page.php';

        /**
         *  Products short code
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-products.php';

        /**
         * Thank you
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-thank-you.php';

        /**
         *  Categories short code
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/class-prodigy-shortcode-categories.php';

        /*
         * theme
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/accordion.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/banner.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/contact_info.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/button.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/divider.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/main_container.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/main_home_container.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/map.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/map_tabs.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/pg_div_row.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/profile.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/row.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/slider.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/text_box.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/shortcodes/theme/text_title_box.php';
        

        ///// Shortcodes /////


        ////////////// Public ///////////////////////////////////////////////////////////////////////

        /**
         * Set product mapper for use in templates
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/class-prodigy-product-template-item.php';

        /**
         * Render attributes in product page
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/pages/class-prodigy-product-page.php';

        /**
         * Render all products page
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/public/pages/class-prodigy-shop-page.php';

        /**
         * Customise twenty-nineteen theme
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/theme-support/class-prodigy-twenty-nineteen.php';

        /**
         * Add wizard
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-wizard.php';

        /**
         * Widgets
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/widgets/class-common-prodigy-widget.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/widgets/class-search-prodigy-widget.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/widgets/class-categories-prodigy-widget.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/widgets/class-my-account-prodigy-widget.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/widgets/class-filters-prodigy-widget.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/widgets/class-active-filters-prodigy-widget.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/widgets/class-cart-prodigy-widget.php';

        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-activation-menu.php';

        /**
         *  Add batch products class
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-batch-products-synchronize.php';


        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once PRODIGY_PLUGIN_PATH . 'templates/class-prodigy-public.php';

        ////////////// Admin ///////////////////////////////////////////////////////////////////////

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */

        /**
         * Abstract admin class
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-main-abstract.php';


        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin.php';
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-tag.php';
        /**
         * TODO
         * Add new category fields
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-category.php';
        /**
         * Add new tags fields
         */


        /**
         * Change admin term sorting
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/prodigy-global-term.php';

        /**
         * Create settings menu
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-settings.php';

        /**
         * Create attributes menu
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-attributes.php';


        /**
         *  Main meta-box class
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-meta-box.php';

        /**
         * Create gallery meta-box
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-gallery-box.php';

        /**
         *  Create product price meta-box
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-price-box.php';

        /**
         *  Create product inventory meta-box
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-inventory-box.php';

        /**
         * Create product shipping meta-box
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-shipping-box.php';

        /**
         *  Create product attributes meta-box
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-attributes-box.php';

        /**
         * Create product variants meta-box
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-variants-box.php';

        /**
         * Create product upsell meta-box
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-related-box.php';

        /**
         *  Add Product list filters
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-list-filters.php';

        

        /**
         * Add columns on product list
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-list-columns.php';

        /**
         *  Update quick edit on product list
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-quick-edit.php';

        /**
         * Update bulk edit
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-bulk-edit.php';

        /**
         * Send product to hosted system
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-product-synchronize.php';
    
        /**
         * Send batches categories
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/batches-content-synchronization/class-prodigy-batches-categories.php';
    
        /**
         * Synchronize data from hosted system
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/batches-content-synchronization/class-prodigy-synchronize-data.php';
        
        /**
         * Set admin notices
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-notices.php';

        /**
         * Set admin notices
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-notices.php';

        /**
         *  Send batch products
         */
        require_once PRODIGY_PLUGIN_PATH . 'includes/batches-content-synchronization/class-prodigy-send-batch-products.php';

        /**
         * Set admin notices
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-notices.php';

        /**
         * Set admin notices
         */
        require_once PRODIGY_PLUGIN_PATH . 'admin/class-prodigy-admin-notices.php';
    
        require_once PRODIGY_PLUGIN_PATH . 'includes/manage-types/class-prodigy-type.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/manage-types/class-prodigy-update-type-products.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/manage-types/class-prodigy-update-type-categories.php';
        require_once PRODIGY_PLUGIN_PATH . 'includes/manage-types/class-prodigy-update-type-tags.php';

        $this->loader = new Prodigy_Loader();
    }

    /**
     *  Set category ajax ordering
     */
    public function set_ajax() {
        Prodigy_Ajax::init();
    }

    /**
     * Update admin crud category
     */
    public function admin_update_category() {
        new Prodigy_Admin_Category();
    }
    
    /**
     * Update admin crud tag
     */
    public function admin_update_tag() {
        new Prodigy_Admin_Tag();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Prodigy_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Prodigy_i18n();

        $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Prodigy_Admin( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Prodigy_Public( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @return    string    The name of the plugin.
     * @since     1.0.0
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @return    Prodigy_Loader    Orchestrates the hooks of the plugin.
     * @since     1.0.0
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @return    string    The version number of the plugin.
     * @since     1.0.0
     */
    public function get_version() {
        return $this->version;
    }

    /**
     * Get prodigy pages list
     *
     * @return array
     */
    public static function get_pages_list() {
        $screen_ids   = [
            self::get_prodigy_product_type(),
            self::get_prodigy_category_type(),
            self::get_prodigy_tag_type(),
            'edit-' . self::get_prodigy_product_type(),
            'edit-' . self::get_prodigy_category_type(),
            'edit-' . self::get_prodigy_tag_type(),
            self::get_prodigy_product_type() . self::PAGE_PRODIGY_ATTRIBUTES,
            self::get_prodigy_product_type() . self::PAGE_PRODIGY_SETTINGS,
        ];

        return $screen_ids;
    }
    
    
    /**
     * Important: You should also have a conditional so this is only set once.
     *
     * You should not flush rewrites on init, never. It is expensive and slows your page load times dramatically.
     * Rewrite rules are saved in the db, so flushing rewrites on init means that there will be a lot of db calls
     * to resave rewrite rules on every pageload
     */
    public static function refresh_permalink() {
        global $wp_rewrite;

        //Write the rule
        $wp_rewrite->set_permalink_structure('/%postname%/');

        //Set the option
        update_option( "rewrite_rules", FALSE );

        //Flush the rules and tell it to write htaccess
        $wp_rewrite->flush_rules( true );
    }

}
