<?php

/**
 * Prodigy Type class
 *
 * @version    1.0.6
 * @package    Prodigy
 * @subpackage Prodigy/includes/manage-types
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 *
 * Class Prodigy_Type
 */
class Prodigy_Type {
    
    private $type;
    
    protected static $exception_slug = [
        'attachment', 'attachment_id', 'author', 'author_name', 'calendar', 'cat', 'category', 'category__and'
        , 'category__in', 'category__not_in', 'category_name', 'comments_per_page', 'comments_popup'
        , 'customize_messenger_channel', 'customized', 'cpage', 'day', 'debug', 'error', 'exact', 'feed', 'fields'
        , 'hour', 'link_category', 'm', 'minute', 'monthnum', 'more', 'name', 'nav_menu', 'nonce', 'nopaging', 'offset'
        , 'order', 'orderby', 'p', 'page', 'page_id', 'paged', 'pagename', 'pb', 'perm', 'post', 'post__in'
        , 'post__not_in', 'post_format', 'post_mime_type', 'post_status', 'post_tag', 'post_type', 'posts'
        , 'posts_per_archive_page', 'posts_per_page', 'preview', 'robots', 's', 'search', 'second', 'sentence'
        , 'showposts', 'static', 'subpost', 'subpost_id', 'tag', 'tag__and', 'tag__in', 'tag__not_in', 'tag_id'
        , 'tag_slug__and', 'tag_slug__in', 'taxonomy', 'tb', 'term', 'theme', 'type', 'w', 'withcomments'
        , 'withoutcomments', 'year'
    ];
    
    private $current_type_option_name = [
        Prodigy::PRODIGY_POST_TYPE_DEFAULT         => [
            'current_slug_name' => Prodigy::PRODIGY_OPTION_NAME_CURRENT_PRODUCT_TYPE,
            'all_types_name'    => Prodigy::PRODIGY_OPTION_NAME_OLD_PRODUCT_TYPES,
            'access_update'     => Prodigy::PRODIGY_ACCESS_UPDATE_TYPE_PRODUCTS_QUEUE,
        ],
        Prodigy::PRODIGY_TAXONOMY_CATEGORY_DEFAULT => [
            'current_slug_name' => Prodigy::PRODIGY_OPTION_NAME_CURRENT_CATEGORY_TYPE,
            'all_types_name'    => Prodigy::PRODIGY_OPTION_NAME_OLD_CATEGORY_TYPES,
            'access_update'     => Prodigy::PRODIGY_ACCESS_UPDATE_TYPE_CATEGORIES_QUEUE,
        ],
        Prodigy::PRODIGY_TAXONOMY_TAG_DEFAULT      => [
            'current_slug_name' => Prodigy::PRODIGY_OPTION_NAME_CURRENT_TAG_TYPE,
            'all_types_name'    => Prodigy::PRODIGY_OPTION_NAME_OLD_TAG_TYPES,
            'access_update'     => Prodigy::PRODIGY_ACCESS_UPDATE_TYPE_TAGS_QUEUE,
        ],
    ];
    
    /**
     * Prodigy_Type constructor.
     *
     * @param string $type
     */
    public function __construct( string $type ) {
        $this->type = $type;
    }

    /**
     * check dynamic prodigy product type
     *
     * @return bool
     */
    public static function is_prodigy_product_post_type(string $type) {
        return  $type === Prodigy::get_prodigy_product_type();
    }
    
    /**
     * get name current type option
     * @return string
     */
    public function get_type() {
        return (string) $this->get_slug_type( $this->current_type_option_name[ $this->type ]['current_slug_name'] );
    }
    
    /**
     * get name current type option
     * @return string
     */
    public function get_name_current_type() {
        return (string) $this->current_type_option_name[ $this->type ]['current_slug_name'];
    }
    
    /**
     * get name all types option
     * @return string
     */
    public function get_name_all_types() {
        return (string) $this->current_type_option_name[ $this->type ]['all_types_name'];
    }
    
    /**
     * get name access update option
     * @return string
     */
    public function get_name_access_update() {
        return (string) $this->current_type_option_name[ $this->type ]['access_update'];
    }
    
    
    /**
     * @param string $slug
     *
     * @return mixed|string|void
     */
    public function get_slug_type( string $slug ) {
        $type         = $this->type;
        $current_type = (string) get_option( $slug );
        
        if ( ! empty( $current_type ) && self::is_valid_name_slug( $current_type ) ) {
            $type = $current_type;
        }
        
        return $type;
    }
    
    /**
     * @param string $type_slug
     *
     * @return bool
     */
    public static function is_valid_name_slug( string $type_slug ) {
        $result = true;
        
        if ( in_array( $type_slug, self::get_list_exception_slug() ) ) {
            $result = false;
        }
        
        return $result;
    }
    
    /**
     * @return mixed
     */
    public static function get_list_exception_slug() {
        return self::$exception_slug;
    }
    
    /**
     *
     * @param string $old_slug_type
     * @param string $new_slug_type
     *
     * @return bool|string if slug was to change success then return string (new slug)
     */
    public function change_prodigy_type( string $old_slug_type, string $new_slug_type ) {
        $result = false;
        
        $list_types = get_option( $this->get_name_all_types() );
    
        $new_slug_type = sanitize_title($new_slug_type);
        
        if ( ! empty( $list_types ) && is_array( $list_types ) && ! empty( $new_slug_type ) && self::is_valid_name_slug( $new_slug_type ) ) {
            array_push( $list_types, $old_slug_type );
            update_option( $this->get_name_all_types(), array_unique( $list_types ) );
            update_option( $this->get_name_current_type(), $new_slug_type );
            update_option( $this->get_name_access_update(), true );
            update_option( Prodigy::PRODIGY_REDIRECT_AFTER_UPDATE_TYPES, true );
            $result = $new_slug_type;
        }
        
        return $result;
    }
    
    /**
     * @param array $list
     */
    public static function registerAllTypes( array $list ) {
        foreach ( $list as $item ) {
            if (Prodigy::get_prodigy_product_type() != $item) {
                self::registerProdigyPostType( $item );
            }
        }
    }
    
    /**
     * @param string $type
     *
     * @return WP_Error|WP_Post_Type
     */
    public static function registerProdigyPostType( string $type ) {
    
        return register_post_type(
            $type,
            [
                'label'        => __( $type, 'prodigy' ),
                'public'       => false,
                'hierarchical' => false,
                'supports'     => false,
                'rewrite'      => false,
            ]
        );
    }
    
    /**
     * @param $product_type
     */
    public static function redirect_to_settings( $product_type ) {
        if ( empty( $product_type ) ) {
            $path_admin = '';
        } else {
            Prodigy_Admin_Notification::displaySuccess( esc_html__( 'Your settings have been saved', 'prodigy' ) );
            $path_admin = 'edit.php?post_type=' . $product_type . '&page=prodigy_settings&tab=product';
        }
        $url = admin_url( $path_admin );
        wp_redirect( $url );
        exit;
    }
    
}



